# Copyright 2020 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The AsyncIO version of the reflection servicer."""

from typing import AsyncIterable

import grpc
from grpc_reflection.v1alpha import reflection_pb2 as _reflection_pb2
from grpc_reflection.v1alpha._base import BaseReflectionServicer


class ReflectionServicer(BaseReflectionServicer):
    """Servicer handling RPCs for service statuses."""

    async def ServerReflectionInfo(
        self,
        request_iterator: AsyncIterable[
            _reflection_pb2.ServerReflectionRequest
        ],
        unused_context,
    ) -> AsyncIterable[_reflection_pb2.ServerReflectionResponse]:
        async for request in request_iterator:
            if request.HasField("file_by_filename"):
                yield self._file_by_filename(request.file_by_filename)
            elif request.HasField("file_containing_symbol"):
                yield self._file_containing_symbol(
                    request.file_containing_symbol
                )
            elif request.HasField("file_containing_extension"):
                yield self._file_containing_extension(
                    request.file_containing_extension.containing_type,
                    request.file_containing_extension.extension_number,
                )
            elif request.HasField("all_extension_numbers_of_type"):
                yield self._all_extension_numbers_of_type(
                    request.all_extension_numbers_of_type
                )
            elif request.HasField("list_services"):
                yield self._list_services()
            else:
                yield _reflection_pb2.ServerReflectionResponse(
                    error_response=_reflection_pb2.ErrorResponse(
                        error_code=grpc.StatusCode.INVALID_ARGUMENT.value[0],
                        error_message=grpc.StatusCode.INVALID_ARGUMENT.value[
                            1
                        ].encode(),
                    )
                )


__all__ = [
    "ReflectionServicer",
]



#!/usr/bin/env python3

# Copyright 2023 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import subprocess

# Maximum path length for a path in the repository before we start seeing
# problems with Windows cloning the repository. (kind of arbitrary, less than
# Windows actual limit, but enough that we avoid problems).
maxlen = 150

errors = 0
for path in subprocess.check_output(["git", "ls-files"]).decode().splitlines():
    if len(path) > maxlen:
        print(f"Path too long: {path}")
        errors += 1

if errors:
    print(f"Found {errors} files with paths longer than {maxlen} characters")
    exit(1)



#!/usr/bin/env python
# Copyright 2016 The Chromium Authors
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
A chain with four possible intermediates with different notBefore and notAfter
dates, for testing path bulding prioritization.
"""

import sys
sys.path += ['../..']

import gencerts

DATE_A = '150101120000Z'
DATE_B = '150102120000Z'
DATE_C = '180101120000Z'
DATE_D = '180102120000Z'


root = gencerts.create_self_signed_root_certificate('Root')
root.set_validity_range(DATE_A, DATE_D)

int_ac = gencerts.create_intermediate_certificate('Intermediate', root)
int_ac.set_validity_range(DATE_A, DATE_C)

int_ad = gencerts.create_intermediate_certificate('Intermediate', root)
int_ad.set_validity_range(DATE_A, DATE_D)
int_ad.set_key(int_ac.get_key())

int_bc = gencerts.create_intermediate_certificate('Intermediate', root)
int_bc.set_validity_range(DATE_B, DATE_C)
int_bc.set_key(int_ac.get_key())

int_bd = gencerts.create_intermediate_certificate('Intermediate', root)
int_bd.set_validity_range(DATE_B, DATE_D)
int_bd.set_key(int_ac.get_key())

target = gencerts.create_end_entity_certificate('Target', int_ac)
target.set_validity_range(DATE_A, DATE_D)


gencerts.write_chain('The root', [root], out_pem='root.pem')
gencerts.write_chain('Intermediate with validity range A..C',
                     [int_ac], out_pem='int_ac.pem')
gencerts.write_chain('Intermediate with validity range A..D',
                     [int_ad], out_pem='int_ad.pem')
gencerts.write_chain('Intermediate with validity range B..C',
                     [int_bc], out_pem='int_bc.pem')
gencerts.write_chain('Intermediate with validity range B..D',
                     [int_bd], out_pem='int_bd.pem')
gencerts.write_chain('The target', [target], out_pem='target.pem')




#!/usr/bin/env python3
# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Manage TCP ports for unit tests; started by run_tests.py"""

from __future__ import print_function

import argparse
import hashlib
import os
import platform
import random
import socket
import sys
import threading
import time

from six.moves.BaseHTTPServer import BaseHTTPRequestHandler
from six.moves.BaseHTTPServer import HTTPServer
from six.moves.socketserver import ThreadingMixIn

# increment this number whenever making a change to ensure that
# the changes are picked up by running CI servers
# note that all changes must be backwards compatible
_MY_VERSION = 21

if len(sys.argv) == 2 and sys.argv[1] == "dump_version":
    print(_MY_VERSION)
    sys.exit(0)

argp = argparse.ArgumentParser(description="Server for httpcli_test")
argp.add_argument("-p", "--port", default=12345, type=int)
argp.add_argument("-l", "--logfile", default=None, type=str)
args = argp.parse_args()

if args.logfile is not None:
    sys.stdin.close()
    sys.stderr.close()
    sys.stdout.close()
    sys.stderr = open(args.logfile, "w")
    sys.stdout = sys.stderr

print("port server running on port %d" % args.port)

pool = []
in_use = {}
mu = threading.Lock()

# Cronet restricts the following ports to be used (see
# https://cs.chromium.org/chromium/src/net/base/port_util.cc). When one of these
# ports is used in a Cronet test, the test would fail (see issue #12149). These
# ports must be excluded from pool.
cronet_restricted_ports = [
    1,
    7,
    9,
    11,
    13,
    15,
    17,
    19,
    20,
    21,
    22,
    23,
    25,
    37,
    42,
    43,
    53,
    77,
    79,
    87,
    95,
    101,
    102,
    103,
    104,
    109,
    110,
    111,
    113,
    115,
    117,
    119,
    123,
    135,
    139,
    143,
    179,
    389,
    465,
    512,
    513,
    514,
    515,
    526,
    530,
    531,
    532,
    540,
    556,
    563,
    587,
    601,
    636,
    993,
    995,
    2049,
    3659,
    4045,
    6000,
    6665,
    6666,
    6667,
    6668,
    6669,
    6697,
]


def can_connect(port):
    # this test is only really useful on unices where SO_REUSE_PORT is available
    # so on Windows, where this test is expensive, skip it
    if platform.system() == "Windows":
        return False
    s = socket.socket()
    try:
        s.connect(("localhost", port))
        return True
    except socket.error as e:
        return False
    finally:
        s.close()


def can_bind(port, proto):
    s = socket.socket(proto, socket.SOCK_STREAM)
    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    try:
        s.bind(("localhost", port))
        return True
    except socket.error as e:
        return False
    finally:
        s.close()


def refill_pool(max_timeout, req):
    """Scan for ports not marked for being in use"""
    chk = [
        port
        for port in range(1025, 32766)
        if port not in cronet_restricted_ports
    ]
    random.shuffle(chk)
    for i in chk:
        if len(pool) > 100:
            break
        if i in in_use:
            age = time.time() - in_use[i]
            if age < max_timeout:
                continue
            req.log_message("kill old request %d" % i)
            del in_use[i]
        if (
            can_bind(i, socket.AF_INET)
            and can_bind(i, socket.AF_INET6)
            and not can_connect(i)
        ):
            req.log_message("found available port %d" % i)
            pool.append(i)


def allocate_port(req):
    global pool
    global in_use
    global mu
    mu.acquire()
    max_timeout = 600
    while not pool:
        refill_pool(max_timeout, req)
        if not pool:
            req.log_message("failed to find ports: retrying soon")
            mu.release()
            time.sleep(1)
            mu.acquire()
            max_timeout /= 2
    port = pool[0]
    pool = pool[1:]
    in_use[port] = time.time()
    mu.release()
    return port


keep_running = True


class Handler(BaseHTTPRequestHandler):
    def setup(self):
        # If the client is unreachable for 5 seconds, close the connection
        self.timeout = 5
        BaseHTTPRequestHandler.setup(self)

    def do_GET(self):
        global keep_running
        global mu
        if self.path == "/get":
            # allocate a new port, it will stay bound for ten minutes and until
            # it's unused
            self.send_response(200)
            self.send_header("Content-Type", "text/plain")
            self.end_headers()
            p = allocate_port(self)
            self.log_message("allocated port %d" % p)
            self.wfile.write(str(p).encode("ascii"))
        elif self.path[0:6] == "/drop/":
            self.send_response(200)
            self.send_header("Content-Type", "text/plain")
            self.end_headers()
            p = int(self.path[6:])
            mu.acquire()
            if p in in_use:
                del in_use[p]
                pool.append(p)
                k = "known"
            else:
                k = "unknown"
            mu.release()
            self.log_message("drop %s port %d" % (k, p))
        elif self.path == "/version_number":
            # fetch a version string and the current process pid
            self.send_response(200)
            self.send_header("Content-Type", "text/plain")
            self.end_headers()
            self.wfile.write(str(_MY_VERSION).encode("ascii"))
        elif self.path == "/dump":
            # yaml module is not installed on Macs and Windows machines by default
            # so we import it lazily (/dump action is only used for debugging)
            import yaml

            self.send_response(200)
            self.send_header("Content-Type", "text/plain")
            self.end_headers()
            mu.acquire()
            now = time.time()
            out = yaml.dump(
                {
                    "pool": pool,
                    "in_use": dict(
                        (k, now - v) for k, v in list(in_use.items())
                    ),
                }
            )
            mu.release()
            self.wfile.write(out.encode("ascii"))
        elif self.path == "/quitquitquit":
            self.send_response(200)
            self.end_headers()
            self.server.shutdown()


class ThreadedHTTPServer(ThreadingMixIn, HTTPServer):
    """Handle requests in a separate thread"""


ThreadedHTTPServer(("", args.port), Handler).serve_forever()



# Copyright 2016 The Chromium Authors
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

'''Generates a test suite from NIST PKITS test descriptions.

The output is a set of Type Parameterized Tests which are included by
pkits_unittest.h. See pkits_unittest.h for information on using the tests.
GoogleTest has a limit of 50 tests per type parameterized testcase, so the tests
are split up by section number (this also makes it possible to easily skip
sections that pertain to non-implemented features).

Usage:
  generate_tests.py <PKITS.pdf> <output.h>
'''

import os
import re
import subprocess
import sys
import tempfile


def sanitize_name(s):
  return s.translate(str.maketrans('', '', ' -'))


def finalize_test_case(test_case_name, sanitized_test_names, output):
  output.write('\nWRAPPED_REGISTER_TYPED_TEST_SUITE_P(%s' % test_case_name)
  for name in sanitized_test_names:
    output.write(',\n    %s' % name)
  output.write(');\n')


def bool_to_str(b):
  return "true" if b else "false"


def make_policies_string(policies):
  return '"' + ','.join(policies) + '"'


def output_test(test_case_name, test_number, raw_test_name, subpart_number,
                info, certs, crls, sanitized_test_names, output):
  '''Writes a test case to |output|, and appends the test name to
  |sanitized_test_names|.'''
  sanitized_test_name = 'Section%s%s' % (test_number.split('.')[1],
                                         sanitize_name(raw_test_name))

  subpart_comment = ''
  if subpart_number is not None:
    sanitized_test_name += "Subpart%d" % (subpart_number)
    subpart_comment = ' (Subpart %d)' % (subpart_number)

  sanitized_test_names.append(sanitized_test_name)

  certs_formatted = ', '.join('"%s"' % n for n in certs)
  crls_formatted = ', '.join('"%s"' % n for n in crls)

  output.write('''
// %(test_number)s %(raw_test_name)s%(subpart_comment)s
WRAPPED_TYPED_TEST_P(%(test_case_name)s, %(sanitized_test_name)s) {
  const char* const certs[] = {
    %(certs_formatted)s
  };
  const char* const crls[] = {
    %(crls_formatted)s
  };
''' % vars())

  default_info = TestInfo(None)

  if info.include_subpart_in_test_number:
    test_number = "%s.%d" % (test_number, subpart_number)

  output.write('''PkitsTestInfo info;
  info.test_number = "%s";
  info.should_validate = %s;
''' % (test_number, bool_to_str(info.should_validate)))

  # Output any non-default inputs/outputs. Only properties that differ from
  # the defaults are written, so as to keep the generated file more readable.
  if info.initial_policy_set != default_info.initial_policy_set:
    output.write('''  info.SetInitialPolicySet(%s);
''' % make_policies_string(info.initial_policy_set))

  if info.initial_explicit_policy != default_info.initial_explicit_policy:
    output.write('''  info.SetInitialExplicitPolicy(%s);
''' % bool_to_str(info.initial_explicit_policy))

  if (info.initial_policy_mapping_inhibit !=
          default_info.initial_policy_mapping_inhibit):
    output.write('''  info.SetInitialPolicyMappingInhibit(%s);
''' % bool_to_str(info.initial_policy_mapping_inhibit))

  if (info.initial_inhibit_any_policy !=
          default_info.initial_inhibit_any_policy):
    output.write('''  info.SetInitialInhibitAnyPolicy(%s);
''' % bool_to_str(info.initial_inhibit_any_policy))

  if (info.user_constrained_policy_set !=
          default_info.user_constrained_policy_set):
    output.write('''  info.SetUserConstrainedPolicySet(%s);
''' % make_policies_string(info.user_constrained_policy_set))

  output.write('''
  this->RunTest(certs, crls, info);
}
''' % vars())


# Matches a section header, ex: "4.1 Signature Verification"
SECTION_MATCHER = re.compile('^\s*(\d+\.\d+)\s+(.+?)\s*\ufffd?$')
# Matches a test header, ex: "4.1.1 Valid Signatures Test1"
TEST_MATCHER = re.compile('^\s*(\d+\.\d+.\d+)\s+(.+?)\s*\ufffd?$')

# Matches the various headers in a test specification.
EXPECTED_HEADER_MATCHER = re.compile('^\s*Expected Result:')
PROCEDURE_HEADER_MATCHER = re.compile('^\s*Procedure:')
PATH_HEADER_MATCHER = re.compile('^\s*Certification Path:')

# Matches the Procedure text if using default settings.
USING_DEFAULT_SETTINGS_MATCHER = re.compile(
    '^.*using the \s*default settings.*')

# Matches the description text if using custom settings.
CUSTOM_SETTINGS_MATCHER = re.compile(
    '.*this\s+test\s+be\s+validated\s+using\s+the\s+following\s+inputs:.*')

# Match an expected test result. Note that some results in the PDF have a typo
# "path not should validate" instead of "path should not validate".
TEST_RESULT_MATCHER = re.compile(
    '^.*path (should validate|should not validate|not should validate)')

# Matches a line in the certification path, ex:
#    "\u2022 Good CA Cert, Good CA CRL"
PATH_MATCHER = re.compile('^\s*\u2022\s*(.+)\s*$')
# Matches a page number. These may appear in the middle of multi-line fields and
# thus need to be ignored.
PAGE_NUMBER_MATCHER = re.compile('^\s*\d+\s*$')
# Matches if an entry in a certification path refers to a CRL, ex:
# "onlySomeReasons CA2 CRL1".
CRL_MATCHER = re.compile('^.*CRL\d*$')


class TestSections(object):
  def __init__(self):
    self.description_lines = []
    self.procedure_lines = []
    self.expected_result_lines = []
    self.cert_path_lines = []


def parse_main_test_sections(lines, i):
  result = TestSections()

  # Read the description lines (text after test name up until
  # "Procedure:").
  result.description_lines = []
  while i < len(lines):
    if PROCEDURE_HEADER_MATCHER.match(lines[i]):
      break
    result.description_lines.append(lines[i])
    i += 1

  # Read the procedure lines (text starting at "Procedure:" and up until
  # "Expected Result:".
  result.procedure_lines = []
  while i < len(lines):
    if EXPECTED_HEADER_MATCHER.match(lines[i]):
      break
    result.procedure_lines.append(lines[i])
    i += 1

  # Read the expected result lines (text starting at "Expected Result:" and up
  # until "Certification Path:".
  result.expected_result_lines = []
  while i < len(lines):
    if PATH_HEADER_MATCHER.match(lines[i]):
      break
    result.expected_result_lines.append(lines[i])
    i += 1

  # Read the certification path lines (text starting at "Certification Path:"
  # and up until the next test title.
  result.cert_path_lines = []
  while i < len(lines):
    if TEST_MATCHER.match(lines[i]) or SECTION_MATCHER.match(lines[i]):
      break
    result.cert_path_lines.append(lines[i])
    i += 1

  return i, result


def parse_cert_path_lines(lines):
  path_lines = []
  crls = []
  certs = []

  for line in lines[1:]:
    line = line.strip()

    if "is composed of the following objects:" in line:
      continue
    if "See the introduction to Section 4.4 for more information." in line:
      continue

    if not line or PAGE_NUMBER_MATCHER.match(line):
      continue
    path_match = PATH_MATCHER.match(line)
    if path_match:
      path_lines.append(path_match.group(1))
      continue
    # Continuation of previous path line.
    path_lines[-1] += ' ' + line

  for path_line in path_lines:
    for path in path_line.split(','):
      path = sanitize_name(path.strip())
      if CRL_MATCHER.match(path):
        crls.append(path)
      else:
        certs.append(path)

  return certs, crls


ANY_POLICY = 'anyPolicy'
TEST_POLICY_1 = 'NIST-test-policy-1'
TEST_POLICY_2 = 'NIST-test-policy-2'
TEST_POLICY_3 = 'NIST-test-policy-3'
TEST_POLICY_6 = 'NIST-test-policy-6'

# Note: This omits some outputs from PKITS:
#
#  * authorities-constrained-policy-set
#  * explicit-policy-indicator
class TestInfo(object):
  """This structure describes a test inputs and outputs"""

  def __init__(self, should_validate,
               # These defaults come from section 3 of PKITS.pdf
               initial_policy_set = [ANY_POLICY],
               initial_explicit_policy = False,
               initial_policy_mapping_inhibit = False,
               initial_inhibit_any_policy = False,
               # In all of the tests that are not related to policy processing,
               # each certificate in the path asserts the certificate policy
               # 2.16.840.1.101.3.2.1.48.1
               user_constrained_policy_set = [TEST_POLICY_1],
               include_subpart_in_test_number = False):
    self.should_validate = should_validate
    self.initial_policy_set = initial_policy_set
    self.initial_explicit_policy = initial_explicit_policy
    self.initial_policy_mapping_inhibit = initial_policy_mapping_inhibit
    self.initial_inhibit_any_policy = initial_inhibit_any_policy
    self.user_constrained_policy_set = user_constrained_policy_set
    self.include_subpart_in_test_number = include_subpart_in_test_number


TEST_OVERRIDES = {
  '4.8.1': [ # All Certificates Same Policy Test1
    # 1. default settings, but with initial-explicit-policy set. The path
    # should validate successfully
    TestInfo(True, initial_explicit_policy=True,
             user_constrained_policy_set=[TEST_POLICY_1]),

    # 2. default settings, but with initial-explicit-policy set and
    # initial-policy-set = {NIST-test-policy-1}. The path should validate
    # successfully.
    TestInfo(True, initial_explicit_policy=True,
             initial_policy_set=[TEST_POLICY_1],
             user_constrained_policy_set=[TEST_POLICY_1]),

    # 3. default settings, but with initial-explicit-policy set and
    # initial-policy-set = {NIST-test-policy-2}. The path should not validate
    # successfully.
    TestInfo(False, initial_explicit_policy=True,
             initial_policy_set=[TEST_POLICY_2],
             user_constrained_policy_set=[]),

    # 4. default settings, but with initial-explicit-policy set and
    # initial-policy-set = {NIST-test-policy-1, NIST-test-policy-2}. The path
    # should validate successfully.
    TestInfo(True, initial_explicit_policy=True,
             initial_policy_set=[TEST_POLICY_1, TEST_POLICY_2],
             user_constrained_policy_set=[TEST_POLICY_1]),
  ],

  '4.8.2': [ # All Certificates No Policies Test2
    # 1. default settings. The path should validate successfully.
    TestInfo(True, user_constrained_policy_set=[]),

    # 2. default settings, but with initial-explicit-policy set. The path
    # should not validate successfully
    TestInfo(False, initial_explicit_policy=True,
             user_constrained_policy_set=[]),
  ],

  '4.8.3': [ # Different Policies Test3
    # 1. default settings. The path should validate successfully.
    TestInfo(True, user_constrained_policy_set=[]),

    # 2. default settings, but with initial-explicit-policy set. The path
    # should not validate successfully.
    TestInfo(False, initial_explicit_policy=True, user_constrained_policy_set=[]),

    # 3. default settings, but with initial-explicit-policy set and
    # initial-policy-set = {NIST-test-policy-1, NIST-test-policy-2}. The path
    # should not validate successfully.
    TestInfo(False, initial_explicit_policy=True,
             initial_policy_set=[TEST_POLICY_1, TEST_POLICY_2],
             user_constrained_policy_set=[]),
  ],

  '4.8.4': [ # Different Policies Test4
    # Procedure: Validate Different Policies Test4 EE using the default
    # settings or open and verify Signed Test Message 6.2.2.69 using the
    # default settings.
    #
    # Expected Result: The authorities-constrained-policy-set and the
    # user-constrained-policy-set will be empty. The explicit-policy-indicator
    # will be set if the application can process the policyConstraints
    # extension. If the application can process the policyConstraints extension
    # then the path should not validate successfully. If the application can
    # not process the policyConstraints extension, then the path should
    # validate successfully.
    TestInfo(False, user_constrained_policy_set=[]),
  ],

  '4.8.5': [ # 4.8.5 Different Policies Test5
    # Procedure: Validate Different Policies Test5 EE using the default
    # settings or open and verify Signed Test Message 6.2.2.70 using the
    # default settings.
    #
    # Expected Result: The authorities-constrained-policy-set and the
    # user-constrained-policy-set will be empty. The explicit-policy-indicator
    # will be set if the application can process the policyConstraints
    # extension. If the application can process the policyConstraints extension
    # then the path should not validate successfully. If the application can
    # not process the policyConstraints extension, then the path should
    # validate successfully
    TestInfo(False, user_constrained_policy_set=[]),
  ],

  '4.8.6': [ # Overlapping Policies Test6
    # 1. default settings. The path should validate successfully.
    TestInfo(True, user_constrained_policy_set=[TEST_POLICY_1]),

    # 2. default settings, but with initial-policy-set = {NIST-test-policy-1}.
    # The path should validate successfully.
    TestInfo(True, initial_policy_set=[TEST_POLICY_1],
             user_constrained_policy_set=[TEST_POLICY_1]),

    # 3. default settings, but with initial-policy-set = {NIST-test-policy-2}.
    # The path should not validate successfully.
    TestInfo(False, initial_policy_set=[TEST_POLICY_2],
             user_constrained_policy_set=[]),
  ],

  '4.8.7': [ # Different Policies Test7
    # Procedure: Validate Different Policies Test7 EE using the default
    # settings or open and verify Signed Test Message 6.2.2.72 using the
    # default settings.
    #
    # Expected Result: The authorities-constrained-policy-set and the
    # user-constrained-policy-set will be empty. If the
    # explicit-policy-indicator will be set if the application can process the
    # policyConstraints extension. If the application can process the
    # policyConstraints extension, then the path should not validate
    # successfully. If the application can not process the policyConstraints
    # extension, then the path should validate successfully.
    TestInfo(False, user_constrained_policy_set=[]),
  ],

  '4.8.8': [ # Different Policies Test8
    # Procedure: Validate Different Policies Test8 EE using the default
    # settings or open and verify Signed Test Message 6.2.2.73 using the
    # default settings.
    #
    # Expected Result: The authorities-constrained-policy-set and the
    # user-constrained-policy-set will be empty. The explicit-policy-indicator
    # will be set if the application can process the policyConstraints
    # extension. If the application can process the policyConstraints extension
    # then the path should not validate successfully. If the application can
    # not process the policyConstraints extension, then the path should
    # validate successfully.
    TestInfo(False, user_constrained_policy_set=[]),
  ],

  '4.8.9': [ # Different Policies Test9
    # Procedure: Validate Different Policies Test9 EE using the default
    # settings or open and verify Signed Test Message 6.2.2.74 using the
    # default settings.
    #
    # Expected Result: The authorities-constrained-policy-set and the
    # user-constrained-policy-set will be empty. The explicit-policy-indicator
    # will be set if the application can process the policyConstraints
    # extension. If the application can process the policyConstraints
    # extension, then the path should not validate successfully. If the
    # application can not process the policyConstraints extension, then the
    # path should validate successfully.
    TestInfo(False, user_constrained_policy_set=[]),
  ],

  '4.8.10': [ # All Certificates Same Policies Test10
    # 1. default settings. The path should validate successfully.
    TestInfo(True, user_constrained_policy_set=[TEST_POLICY_1, TEST_POLICY_2]),

    # 2. default settings, but with initial-policy-set = {NIST-test-policy-1}.
    # The path should validate successfully.
    TestInfo(True, initial_policy_set=[TEST_POLICY_1],
             user_constrained_policy_set=[TEST_POLICY_1]),

    # 3. default settings, but with initial-policy-set = {NIST-test-policy-2}.
    # The path should validate successfully.
    TestInfo(True, initial_policy_set=[TEST_POLICY_2],
             user_constrained_policy_set=[TEST_POLICY_2]),
  ],

  '4.8.11': [ # All Certificates AnyPolicy Test11
    # 1. default settings. The path should validate successfully.
    TestInfo(True, user_constrained_policy_set=[ANY_POLICY]),

    # 2. default settings, but with initial-policy-set = {NIST-test-policy-1}.
    # The path should validate successfully.
    TestInfo(True, initial_policy_set=[TEST_POLICY_1],
             user_constrained_policy_set=[TEST_POLICY_1]),
  ],

  '4.8.12': [ # Different Policies Test12
    # Procedure: Validate Different Policies Test12 EE using the default
    # settings or open and verify Signed Test Message 6.2.2.77 using the
    # default settings.
    #
    # Expected Result: The authorities-constrained-policy-set and the
    # user-constrained-policy-set will be empty. The explicit-policy-indicator
    # will be set if the application can process the policyConstraints
    # extension. If the application can process the policyConstraints
    # extension, then the path should not validate successfully. If the
    # application can not process the policyConstraints extension, then the
    # path should validate successfully.
    TestInfo(False, user_constrained_policy_set=[]),
  ],

  '4.8.13': [ # All Certificates Same Policies Test13
    # 1. default settings, but with initial-policy-set = {NIST-test-policy-1}.
    # The path should validate successfully.
    TestInfo(True, initial_policy_set=[TEST_POLICY_1],
             user_constrained_policy_set=[TEST_POLICY_1]),

    # 2. default settings, but with initial-policy-set = {NIST-test-policy-2}.
    # The path should validate successfully.
    TestInfo(True, initial_policy_set=[TEST_POLICY_2],
             user_constrained_policy_set=[TEST_POLICY_2]),

    # 3. default settings, but with initial-policy-set = {NIST-test-policy-3}.
    # The path should validate successfully.
    TestInfo(True, initial_policy_set=[TEST_POLICY_3],
             user_constrained_policy_set=[TEST_POLICY_3]),
  ],

  '4.8.14': [ # AnyPolicy Test14
    # 1. default settings, but with initial-policy-set = {NIST-test-policy-1}.
    # The path should validate successfully.
    TestInfo(True, initial_policy_set=[TEST_POLICY_1],
             user_constrained_policy_set=[TEST_POLICY_1]),

    # 2. default settings, but with initial-policy-set = {NIST-test-policy-2}.
    # The path should not validate successfully.
    TestInfo(False, initial_policy_set=[TEST_POLICY_2],
             user_constrained_policy_set=[]),
  ],

  '4.8.15': [ # User Notice Qualifier Test15
    # Procedure: Validate User Notice Qualifier Test15 EE using the default
    # settings or open and verify Signed Test Message 6.2.2.80 using the
    # default settings.
    #
    # Expected Result: The authorities-constrained-policy-set will be
    # {NIST-test-policy-1} and the explicit-policy-indicator will be the same
    # as the initial-explicit-policy indicator. If the initial-policy-set is
    # any-policy or otherwise includes NIST-test-policy-1, then the
    # user-constrained-policy-set will be {NIST-test-policy-1}. If not, the
    # user-constrained-policy-set will be empty. If the initial-explicit-policy
    # indicator is set and the initial-policy-set does not include
    # NIST-test-policy-1, then the path should be rejected, otherwise it should
    # validate successfully. If the path validates successfully, then the
    # application should display the user notice.
    TestInfo(True, user_constrained_policy_set=[TEST_POLICY_1]),
  ],

  '4.8.16': [ # User Notice Qualifier Test16
    # Procedure: Validate User Notice Qualifier Test16 EE using the default
    # settings or open and verify Signed Test Message 6.2.2.81 using the
    # default settings.
    #
    # Expected Result: The authorities-constrained-policy-set will be
    # {NIST-test-policy-1} and the explicit-policy-indicator will be the same
    # as the initial-explicit-policy indicator. If the initial-policy-set is
    # any-policy or otherwise includes NIST-test-policy-1, then the
    # user-constrained-policy-set will be {NIST-test-policy-1}. If not, the
    # user-constrained-policy-set will be empty. If the initial-explicit-policy
    # indicator is set and the initial-policy-set does not include
    # NIST-test-policy-1, then the path should be rejected, otherwise it should
    # validate successfully. If the path validates successfully, then the
    # application should display the user notice associated with
    # NIST-test-policy-1. The user notice associated with NIST-test-policy-2
    # should not be displayed.
    TestInfo(True, user_constrained_policy_set=[TEST_POLICY_1]),
  ],

  '4.8.17': [ # User Notice Qualifier Test17
    # Procedure: Validate User Notice Qualifier Test17 EE using the default
    # settings or open and verify Signed Test Message 6.2.2.82 using the
    # default settings.
    #
    # Expected Result: The authorities-constrained-policy-set will be
    # {NIST-test-policy-1} and the explicit-policy-indicator will be the same
    # as the initial-explicit-policy indicator. If the initial-policy-set is
    # any-policy or otherwise includes NIST-test-policy-1, then the
    # user-constrained-policy-set will be {NIST-test-policy-1}. If not, the
    # user-constrained-policy-set will be empty. If the initial-explicit-policy
    # indicator is set and the initial-policy-set does not include
    # NIST-test-policy-1, then the path should be rejected, otherwise it should
    # validate successfully. If the path validates successfully, then the
    # application should display the user notice associated with anyPolicy.
    TestInfo(True, user_constrained_policy_set=[TEST_POLICY_1]),
  ],

  '4.8.18': [ # User Notice Qualifier Test18
    # 1. default settings, but with initial-policy-set = {NIST-test-policy-1}.
    # The path should validate successfully and the qualifier associated with
    # NIST-test-policy-1 in the end entity certificate should be displayed.
    TestInfo(True, initial_policy_set=[TEST_POLICY_1],
             user_constrained_policy_set=[TEST_POLICY_1]),

    # 2. default settings, but with initial-policy-set = {NIST-test-policy-2}.
    # The path should validate successfully and the qualifier associated with
    # anyPolicy in the end entity certificate should be displayed.
    TestInfo(True, initial_policy_set=[TEST_POLICY_2],
             user_constrained_policy_set=[TEST_POLICY_2]),
  ],

  '4.8.19': [ # User Notice Qualifier Test19
    # Procedure: Validate User Notice Qualifier Test19 EE using the default
    # settings or open and verify Signed Test Message 6.2.2.84 using the
    # default settings.
    #
    # Expected Result: The authorities-constrained-policy-set will be
    # {NIST-test-policy-1} and the explicit-policy-indicator will be the same
    # as the initial-explicit-policy indicator. If the initial-policy-set is
    # any-policy or otherwise includes NIST-test-policy-1, then the
    # user-constrained-policy-set will be {NIST-test-policy-1}. If not, the
    # user-constrained-policy-set will be empty. If the initial-explicit-policy
    # indicator is set and the initial-policy-set does not include
    # NIST-test-policy-1, then the path should be rejected, otherwise it should
    # validate successfully.  Since the explicitText exceeds the maximum size
    # of 200 characters, the application may choose to reject the certificate.
    # If the application accepts the certificate, display of the user notice is
    # optional.
    TestInfo(True, user_constrained_policy_set=[TEST_POLICY_1]),
  ],

  '4.8.20': [ # CPS Pointer Qualifier Test20
    # Procedure: Validate CPS Pointer Qualifier Test20 EE using the default
    # settings or open and verify Signed Test Message 6.2.2.85 using the
    # default settings. (If possible, it is recommended that this test be run
    # with the initial-explicit-policy indicator set. If this can not be done,
    # manually check that the authorities-constrained-policy-set and
    # user-constrained-policy-set are correct.)
    #
    # Expected Result: The authorities-constrained-policy-set will be
    # {NIST-test-policy-1} and the explicit-policy-indicator will be the same
    # as the initial-explicit-policy indicator. If the initial-policy-set is
    # any-policy or otherwise includes NIST-test-policy-1, then the
    # user-constrained-policy-set will be {NIST-test-policy-1}. If not, the
    # user-constrained-policy-set will be empty. If the initial-explicit-policy
    # indicator is set and the initial-policy-set does not include
    # NIST-test-policy-1, then the path should be rejected, otherwise it should
    # validate successfully. The CPS pointer in the qualifier should be
    # associated with NIST-testpolicy-1 in the
    # authorities-constrained-policy-set (and in the user-constrained-policy-set
    # if NIST-test-policy-1 is in that set). There are no processing
    # requirements associated with the CPS pointer qualifier.
    TestInfo(True, initial_explicit_policy=True,
             initial_policy_set=[TEST_POLICY_1],
             user_constrained_policy_set=[TEST_POLICY_1]),
  ],

  '4.9.1': [ # Valid RequireExplicitPolicy Test1
    # Procedure: Validate Valid requireExplicitPolicy Test1 EE using the
    # default settings or open and verify Signed Test Message 6.2.2.86 using
    # the default settings.
    #
    # Expected Result: The path should validate successfully since the
    # explicit-policy-indicator is not set.
    TestInfo(True, user_constrained_policy_set=[]),
  ],

  '4.9.2': [ # Valid RequireExplicitPolicy Test2
    # Procedure: Validate Valid requireExplicitPolicy Test2 EE using the
    # default settings or open and verify Signed Test Message 6.2.2.87 using
    # the default settings.
    #
    # Expected Result: The path should validate successfully since the
    # explicit-policy-indicator is not set
    TestInfo(True, user_constrained_policy_set=[]),
  ],

  '4.9.6': [ # Valid Self-Issued requireExplicitPolicy Test6
    # Procedure: Validate Valid Self-Issued requireExplicitPolicy Test6 EE using
    # the default settings or open and verify Signed Test Message 6.2.2.91 using
    # the default settings.
    #
    # Expected Result: The path should validate successfully since the
    # explicit-policy-indicator is not set.
    TestInfo(True, user_constrained_policy_set=[]),
  ],

  '4.10.1': [ # Valid Policy Mapping Test1
    # The errors in subparts 2 and 3 vary slightly, so we set
    # include_subpart_in_test_number.

    # 1. default settings, but with initial-policy-set = {NIST-test-policy-1}.
    # The path should validate successfully.
    TestInfo(True, initial_policy_set=[TEST_POLICY_1],
             user_constrained_policy_set=[TEST_POLICY_1],
             include_subpart_in_test_number=True),

    # 2. default settings, but with initial-policy-set = {NIST-test-policy-2}.
    # The path should not validate successfully.
    TestInfo(False, initial_policy_set=[TEST_POLICY_2],
             user_constrained_policy_set=[],
             include_subpart_in_test_number=True),

    # 3. default settings, but with initial-policy-mapping-inhibit set. The
    # path should not validate successfully.
    TestInfo(False, initial_policy_mapping_inhibit=True,
             user_constrained_policy_set=[],
             include_subpart_in_test_number=True),
  ],

  '4.10.2': [ # Invalid Policy Mapping Test2
    # 1. default settings. The path should not validate successfully.
    TestInfo(False, user_constrained_policy_set=[]),

    # 2. default settings, but with initial-policy-mapping-inhibit set. The
    # path should not validate successfully.
    TestInfo(False, initial_policy_mapping_inhibit=True,
             user_constrained_policy_set=[]),
  ],

  '4.10.3': [ # Valid Policy Mapping Test3
    # 1. default settings, but with initial-policy-set = {NIST-test-policy-1}.
    # The path should not validate successfully.
    TestInfo(False, initial_policy_set=[TEST_POLICY_1],
             user_constrained_policy_set=[]),

    # 2. default settings, but with initial-policy-set = {NIST-test-policy-2}.
    # The path should validate successfully.
    TestInfo(True, initial_policy_set=[TEST_POLICY_2],
             user_constrained_policy_set=[TEST_POLICY_2]),
  ],

  '4.10.4': [ # Invalid Policy Mapping Test4
    # Procedure: Validate Invalid Policy Mapping Test4 EE using the default
    # settings or open and verify Signed Test Message 6.2.2.97 using the
    # default settings.
    #
    # Expected Result: The authorities-constrained-policy-set and the
    # user-constrained-policy-set will be empty and the
    # explicit-policy-indicator will be set (if the application can process the
    # policyConstraints extension). If the application can process the
    # policyConstraints extension, then the path should be rejected, otherwise
    # it should validate successfully.
    TestInfo(False, user_constrained_policy_set=[]),
  ],

  '4.10.5': [ # Valid Policy Mapping Test5
    # 1. default settings, but with initial-policy-set = {NIST-test-policy-1}.
    # The path should validate successfully.
    TestInfo(True, initial_policy_set=[TEST_POLICY_1],
             user_constrained_policy_set=[TEST_POLICY_1]),

    # 2. default settings, but with initial-policy-set = {NIST-test-policy-6}.
    # The path should not validate successfully.
    TestInfo(False, initial_policy_set=[TEST_POLICY_6],
             user_constrained_policy_set=[]),
  ],

  '4.10.6': [ # Valid Policy Mapping Test6
    # 1. default settings, but with initial-policy-set = {NIST-test-policy-1}.
    # The path should validate successfully.
    TestInfo(True, initial_policy_set=[TEST_POLICY_1],
                   user_constrained_policy_set=[TEST_POLICY_1]),

    # 2. default settings, but with initial-policy-set = {NIST-test-policy-6}.
    # The path should not validate successfully.
    TestInfo(False, initial_policy_set=[TEST_POLICY_6],
             user_constrained_policy_set=[]),
  ],

  '4.10.7': [ # Invalid Mapping From anyPolicy Test7
    # Procedure: Validate Invalid Mapping From anyPolicy Test7 EE using the
    # default settings or open and verify Signed Test Message 6.2.2.100 using
    # the default settings.
    #
    # Expected Result: The path should not validate successfully since the
    # intermediate certificate includes a policy mapping extension in which
    # anyPolicy appears as an issuerDomainPolicy.
    TestInfo(False, user_constrained_policy_set=[]),
  ],

  '4.10.8': [ # Invalid Mapping To anyPolicy Test8
    # Procedure: Validate Invalid Mapping To anyPolicy Test8 EE using the
    # default settings or open and verify Signed Test Message 6.2.2.101 using
    # the default settings.
    #
    # Expected Result: The path should not validate successfully since the
    # intermediate certificate includes a policy mapping extension in which
    # anyPolicy appears as an subjectDomainPolicy.
    TestInfo(False, user_constrained_policy_set=[]),
  ],

  '4.10.9': [ # Valid Policy Mapping Test9
    # Procedure: Validate Valid Policy Mapping Test9 EE using the default
    # settings or open and verify Signed Test Message 6.2.2.102 using the
    # default settings.
    #
    # Expected Result: The authorities-constrained-policy-set will be
    # {NIST-test-policy-1} and the explicit-policy-indicator will be set (if
    # the application can process the policyConstraints extension). If the
    # initial-policy-set is any-policy or otherwise includes
    # NIST-test-policy-1, then the user-constrained-policy-set will be
    # {NIST-test-policy-1}. If not, the user-constrained-policy-set will be
    # empty. If the initial-policy-set does not include NIST-test-policy-1 (and
    # the application can process the policyConstraints extension), then the
    # path should be rejected, otherwise it should validate successfully.
    TestInfo(True),
  ],

  '4.10.10': [ # Invalid Policy Mapping Test10
    # Procedure: Validate Invalid Policy Mapping Test10 EE using the default
    # settings or open and verify Signed Test Message 6.2.2.103 using the
    # default settings.
    #
    # Expected Result: The authorities-constrained-policy-set and the
    # user-constrained-policy-set will be empty and the
    # explicit-policy-indicator will be set (if the application can process the
    # policyConstraints extension). If the application can process the
    # policyConstraints extension, then the path should be rejected, otherwise
    # it should validate successfully.
    TestInfo(False, user_constrained_policy_set=[]),
  ],

  '4.10.11': [ # Valid Policy Mapping Test11
    # Procedure: Validate Valid Policy Mapping Test11 EE using the default
    # settings or open and verify Signed Test Message 6.2.2.104 using the
    # default settings.
    #
    # Expected Result: The authorities-constrained-policy-set will be
    # {NIST-test-policy-1} and the explicit-policy-indicator will be set (if
    # the application can process the policyConstraints extension). If the
    # initial-policy-set is any-policy or otherwise includes
    # NIST-test-policy-1, then the user-constrained-policy-set will be
    # {NIST-test-policy-1}. If not, the user-constrained-policy-set will be
    # empty. If the initial-policy-set does not include NIST-test-policy-1 (and
    # the application can process the policyConstraints extension), then the
    # path should be rejected, otherwise it should validate successfully.
    TestInfo(True, user_constrained_policy_set=[TEST_POLICY_1]),
  ],

  '4.10.12': [ # Valid Policy Mapping Test12
    # 1. default settings, but with initial-policy-set = {NIST-test-policy-1}.
    # The path should validate successfully and the application should display
    # the user notice associated with NIST-test-policy-3 in the end entity
    # certificate.
    TestInfo(True, initial_policy_set=[TEST_POLICY_1],
             user_constrained_policy_set=[TEST_POLICY_1]),

    # 2. default settings, but with initial-policy-set = {NIST-test-policy-2}.
    # The path should validate successfully and the application should display
    # the user notice associated with anyPolicy in the end entity certificate.
    TestInfo(True, initial_policy_set=[TEST_POLICY_2],
             user_constrained_policy_set=[TEST_POLICY_2]),
  ],

  '4.10.13': [ # Valid Policy Mapping Test13
    # Procedure: Validate Valid Policy Mapping Test13 EE using the default
    # settings or open and verify Signed Test Message 6.2.2.106 using the
    # default settings.
    #
    # Expected Result: The authorities-constrained-policy-set will be
    # {NIST-test-policy-1} and the explicit-policy-indicator will be set (if
    # the application can process the policyConstraints extension). If the
    # initial-policy-set is any-policy or otherwise includes
    # NIST-test-policy-1, then the user-constrained-policy-set will be
    # {NIST-test-policy-1}. If not, the user-constrained-policy-set will be
    # empty. If the initial-policy-set does not include NIST-test-policy-1 (and
    # the application can process the policyConstraints extension), then the
    # path should be rejected, otherwise it should validate successfully. If
    # the path is accepted, the application should display the user notice
    # associated with NIST-testpolicy-1 in the intermediate certificate.
    TestInfo(True, user_constrained_policy_set=[TEST_POLICY_1]),

    # While not explicitly divided into sub-parts, the above describes what
    # should happen given various values of initial-policy-set. Test some
    # combinations, as these cover an interesting interaction with anyPolicy.
    #
    # These extra tests are a regression test for https://crbug.com/1403258.
    TestInfo(True, initial_policy_set=[TEST_POLICY_1, TEST_POLICY_2],
             user_constrained_policy_set=[TEST_POLICY_1]),
    TestInfo(False, initial_policy_set=[TEST_POLICY_2],
             user_constrained_policy_set=[]),
  ],

  '4.10.14': [ # Valid Policy Mapping Test14
    # Procedure: Validate Valid Policy Mapping Test14 EE using the default
    # settings or open and verify Signed Test Message 6.2.2.107 using the
    # default settings.
    #
    # Expected Result: The authorities-constrained-policy-set will be
    # {NIST-test-policy-1} and the explicit-policy-indicator will be set (if
    # the application can process the policyConstraints extension). If the
    # initial-policy-set is any-policy or otherwise includes
    # NIST-test-policy-1, then the user-constrained-policy-set will be
    # {NIST-test-policy-1}. If not, the user-constrained-policy-set will be
    # empty. If the initial-policy-set does not include NIST-test-policy-1 (and
    # the application can process the policyConstraints extension), then the
    # path should be rejected, otherwise it should validate successfully. If
    # the path is accepted, the application should display the user notice
    # associated with anyPolicy in the intermediate certificate
    TestInfo(True, user_constrained_policy_set=[TEST_POLICY_1]),
  ],

  '4.11.1': [ # Invalid inhibitPolicyMapping Test1
    # Procedure: Validate Invalid inhibitPolicyMapping Test1 EE using the
    # default settings or open and verify Signed Test Message 6.2.2.108 using
    # the default settings.
    #
    # Expected Result: The authorities-constrained-policy-set and the
    # user-constrained-policy-set will be empty. The explicit-policy-indicator
    # will be set.  The path should not validate successfully.
    TestInfo(False, user_constrained_policy_set=[]),
  ],

  '4.11.2': [ # Valid inhibitPolicyMapping Test2
    # Procedure: Validate Valid inhibitPolicyMapping Test2 EE using the default
    # settings or open and verify Signed Test Message 6.2.2.109 using the
    # default settings.
    #
    # Expected Result: The authorities-constrained-policy-set will be
    # {NIST-test-policy-1} and the explicit-policy-indicator will be set. If
    # the initial-policy-set is any-policy or otherwise includes
    # NIST-test-policy-1, then the path should validate successfully.
    TestInfo(True, user_constrained_policy_set=[TEST_POLICY_1]),
  ],

  '4.11.3': [ # Invalid inhibitPolicyMapping Test3
    # Procedure: Validate Invalid inhibitPolicyMapping Test3 EE using the
    # default settings or open and verify Signed Test Message 6.2.2.110 using
    # the default settings.
    #
    # Expected Result: The authorities-constrained-policy-set and the
    # user-constrained-policy-set will be empty and the
    # explicit-policy-indicator will be set.  The path should not validate
    # successfully.
    TestInfo(False, user_constrained_policy_set=[]),
  ],

  '4.11.4': [ # Valid inhibitPolicyMapping Test4
    # Procedure: Validate Valid inhibitPolicyMapping Test4 EE using the default
    # settings or open and verify Signed Test Message 6.2.2.111 using the
    # default settings.
    #
    # Expected Result: The authorities-constrained-policy-set will be
    # {NIST-test-policy-2} and the explicit-policy-indicator will be set. If
    # the initial-policy-set is any-policy or otherwise includes
    # NIST-test-policy-2, then the path should validate successfully.
    TestInfo(True, user_constrained_policy_set=[TEST_POLICY_2]),
  ],

  '4.11.5': [ # Invalid inhibitPolicyMapping Test5
    # Procedure: Validate Invalid inhibitPolicyMapping Test5 EE using the
    # default settings or open and verify Signed Test Message 6.2.2.112 using
    # the default settings.
    #
    # Expected Result: The authorities-constrained-policy-set and the
    # user-constrained-policy-set will be empty and the
    # explicit-policy-indicator will be set.  The path should not validate
    # successfully.
    TestInfo(False, user_constrained_policy_set=[]),
  ],

  '4.11.6': [ # Invalid inhibitPolicyMapping Test6
    # Procedure: Validate Invalid inhibitPolicyMapping Test6 EE using the
    # default settings or open and verify Signed Test Message 6.2.2.113 using
    # the default settings.
    #
    # Expected Result: The authorities-constrained-policy-set and the
    # user-constrained-policy-set will be empty and the
    # explicit-policy-indicator will be set. The path should not validate
    # successfully.
    TestInfo(False, user_constrained_policy_set=[]),
  ],

  '4.11.7': [ # Valid Self-Issued inhibitPolicyMapping Test7
    # Procedure: Validate Valid Self-Issued inhibitPolicyMapping Test7 EE using
    # the default settings or open and verify Signed Test Message 6.2.2.114
    # using the default settings.
    #
    # Expected Result: The authorities-constrained-policy-set will be
    # {NIST-test-policy-1} and the explicit-policy-indicator will be set. If
    # the initial-policy-set is any-policy or otherwise includes
    # NIST-test-policy-1, then the path should validate successfully.
    TestInfo(True, user_constrained_policy_set=[TEST_POLICY_1]),
  ],

  '4.11.8': [ # Invalid Self-Issued inhibitPolicyMapping Test8
    # Procedure: Validate Invalid Self-Issued inhibitPolicyMapping Test8 EE
    # using the default settings or open and verify Signed Test Message
    # 6.2.2.115 using the default settings.
    #
    # Expected Result: The authorities-constrained-policy-set and
    # user-constrained-policy-set will be empty and the
    # explicit-policy-indicator will be set. The path should not validate
    # successfully.
    TestInfo(False, user_constrained_policy_set=[]),
  ],

  '4.11.9': [ # Invalid Self-Issued inhibitPolicyMapping Test9
    # Procedure: Validate Invalid Self-Issued inhibitPolicyMapping Test9 EE
    # using the default settings or open and verify Signed Test Message
    # 6.2.2.116 using the default settings.
    #
    # Expected Result: The authorities-constrained-policy-set and
    # user-constrained-policy-set will be empty and the
    # explicit-policy-indicator will be set. The path should not validate
    # successfully.
    TestInfo(False, user_constrained_policy_set=[]),
  ],

  '4.11.10': [ # Invalid Self-Issued inhibitPolicyMapping Test10
    # Procedure: Validate Invalid Self-Issued inhibitPolicyMapping Test10 EE
    # using the default settings or open and verify Signed Test Message
    # 6.2.2.117 using the default settings.
    #
    # Expected Result: The authorities-constrained-policy-set and
    # user-constrained-policy-set will be empty and the
    # explicit-policy-indicator will be set. The path should not validate
    # successfully.
    TestInfo(False, user_constrained_policy_set=[]),
  ],

  '4.11.11': [ # Invalid Self-Issued inhibitPolicyMapping Test11
    # Procedure: Validate Invalid Self-Issued inhibitPolicyMapping Test11 EE
    # using the default settings or open and verify Signed Test Message
    # 6.2.2.118 using the default settings.
    #
    # Expected Result: The authorities-constrained-policy-set and
    # user-constrained-policy-set will be empty and the
    # explicit-policy-indicator will be set. The path should not validate
    # successfully.
    TestInfo(False, user_constrained_policy_set=[]),
  ],

  '4.12.1': [ # Invalid inhibitAnyPolicy Test1
    # Procedure: Validate Invalid inhibitAnyPolicy Test1 EE using the default
    # settings or open and verify Signed Test Message 6.2.2.119 using the
    # default settings.
    #
    # Expected Result: The authorities-constrained-policy-set and
    # user-constrained-policy-set will be empty and the
    # explicit-policy-indicator will be set (if the application can process the
    # policyConstraints extension). If the application can process the
    # policyConstraints extension, then the path should not validate
    # successfully.
    TestInfo(False, user_constrained_policy_set=[]),
  ],

  '4.12.2': [ # Valid inhibitAnyPolicy Test2
    # Procedure: Validate Valid inhibitAnyPolicy Test2 EE using the default
    # settings or open and verify Signed Test Message 6.2.2.120 using the
    # default settings.
    #
    # Expected Result: The authorities-constrained-policy-set will be
    # {NIST-test-policy-1} and the explicit-policy-indicator will be set (if
    # the application can process the policyConstraints extension). If the
    # initial-policy-set is any-policy or otherwise includes
    # NIST-test-policy-1, then the user-constrained-policy-set will be
    # {NIST-test-policy-1} and the path should validate successfully. If not,
    # then the user-constrained-policy-set will be empty. If the
    # user-constrained-policy-set is empty and the application can process the
    # policyConstraints extension, then the path should not validate
    # successfully.
    TestInfo(True, user_constrained_policy_set=[TEST_POLICY_1]),
  ],

  '4.12.3': [ # inhibitAnyPolicy Test3
     # 1. default settings. The path should validate successfully.
    TestInfo(True, user_constrained_policy_set=[TEST_POLICY_1]),

     # 2. default settings, but with initial-inhibit-any-policy set. The path
     # should not validate successfully.
    TestInfo(False, initial_inhibit_any_policy=True,
             user_constrained_policy_set=[]),
  ],

  '4.12.4': [ # Invalid inhibitAnyPolicy Test4
    # Procedure: Validate Invalid inhibitAnyPolicy Test4 EE using the default
    # settings or open and verify Signed Test Message 6.2.2.122 using the
    # default settings.
    #
    # Expected Result: The authorities-constrained-policy-set and
    # user-constrained-policy-set will be empty and the
    # explicit-policy-indicator will be set (if the application can process the
    # policyConstraints extension). If the application can process the
    # policyConstraints extension, then the path should not validate
    # successfully.
    TestInfo(False, user_constrained_policy_set=[]),
  ],

  '4.12.5': [ # Invalid inhibitAnyPolicy Test5
    # Procedure: Validate Invalid inhibitAnyPolicy Test5 EE using the default
    # settings or open and verify Signed Test Message 6.2.2.123 using the
    # default settings.
    #
    # Expected Result: The authorities-constrained-policy-set and
    # user-constrained-policy-set will be empty and the
    # explicit-policy-indicator will be set (if the application can process the
    # policyConstraints extension). If the application can process the
    # policyConstraints extension, then the path should not validate
    # successfully.
    TestInfo(False, user_constrained_policy_set=[]),
  ],

  '4.12.6': [ # Invalid inhibitAnyPolicy Test6
    # Procedure: Validate Invalid inhibitAnyPolicy Test6 EE using the default
    # settings or open and verify Signed Test Message 6.2.2.124 using the
    # default settings.
    #
    # Expected Result: The authorities-constrained-policy-set and
    # user-constrained-policy-set will be empty and the
    # explicit-policy-indicator will be set (if the application can process the
    # policyConstraints extension). If the application can process the
    # policyConstraints extension, then the path should not validate
    # successfully.
    TestInfo(False, user_constrained_policy_set=[]),
  ],

  '4.12.7': [ # Valid Self-Issued inhibitAnyPolicy Test7
    # Procedure: Validate Valid Self-Issued inhibitAnyPolicy Test7 EE using the
    # default settings or open and verify Signed Test Message 6.2.2.125 using
    # the default settings.
    #
    # Expected Result: The authorities-constrained-policy-set will be
    # {NIST-test-policy-1} and the explicit-policy-indicator will be set (if
    # the application can process the policyConstraints extension). If the
    # initial-policy-set is any-policy or otherwise includes
    # NIST-test-policy-1, then the user-constrained-policy-set will be
    # {NIST-test-policy-1} and the path should validate successfully. If not,
    # then the user-constrained-policy-set will be empty. If the
    # user-constrained-policy-set is empty and the application can process the
    # policyConstraints extension, then the path should not validate
    # successfully.
    TestInfo(True, user_constrained_policy_set=[TEST_POLICY_1]),
  ],

  '4.12.8': [ # Invalid Self-Issued inhibitAnyPolicy Test8
    # Procedure: Validate Invalid Self-Issued inhibitAnyPolicy Test8 EE using
    # the default settings or open and verify Signed Test Message 6.2.2.126
    # using the default settings.
    #
    # Expected Result: The authorities-constrained-policy-set and
    # user-constrained-policy-set will be empty and the
    # explicit-policy-indicator will be set (if the application can process the
    # policyConstraints extension). If the application can process the
    # policyConstraints extension, then the path should not validate
    # successfully.
    TestInfo(False, user_constrained_policy_set=[]),
  ],

  '4.12.9': [ # Valid Self-Issued inhibitAnyPolicy Test9
    # Procedure: Validate Valid Self-Issued inhibitAnyPolicy Test9 EE using the
    # default settings or open and verify Signed Test Message 6.2.2.127 using
    # the default settings.
    #
    # Expected Result: The authorities-constrained-policy-set will be
    # {NIST-test-policy-1} and the explicit-policy-indicator will be set (if
    # the application can process the policyConstraints extension). If the
    # initial-policy-set is any-policy or otherwise includes
    # NIST-test-policy-1, then the user-constrained-policy-set will be
    # {NIST-test-policy-1} and the path should validate successfully. If not,
    # then the user-constrained-policy-set will be empty. If the
    # user-constrained-policy-set is empty and the application can process the
    # policyConstraints extension, then the path should not validate
    # successfully.
    TestInfo(True, user_constrained_policy_set=[TEST_POLICY_1]),
  ],

  '4.12.10': [ # Invalid Self-Issued inhibitAnyPolicy Test10
    # Procedure: Validate Invalid Self-Issued inhibitAnyPolicy Test10 EE using
    # the default settings or open and verify Signed Test Message 6.2.2.128
    # using the default settings.
    #
    # Expected Result: The authorities-constrained-policy-set and
    # user-constrained-policy-set will be empty and the
    # explicit-policy-indicator will be set (if the application can process the
    # policyConstraints extension). If the application can process the
    # policyConstraints extension, then the path should not validate
    # successfully.
    TestInfo(False, user_constrained_policy_set=[]),
  ],
}


def parse_test(lines, i, test_case_name, test_number, test_name,
               sanitized_test_names, output):
  # Start by doing a coarse level of parsing that separates out the lines for
  # the main sections.
  i, test_sections = parse_main_test_sections(lines, i)

  certs, crls = parse_cert_path_lines(test_sections.cert_path_lines)

  # Most tests have a formulaic specification: they use the default
  # settings, and have one expectation. These are easily parsed and are handled
  # programmatically. In contrast, many of the policies tests have a more
  # complicated specification which involves multiple subtests having various
  # settings, as well as expectations described in terms of supported
  # extensions. Rather than try to handle all the nuanced language, these are
  # handled manually via "overrides".
  overrides = TEST_OVERRIDES.get(test_number, None)

  if overrides is None:
    # Verify that the test description doesn't include numbered subparts (those
    # are not handled here).
    if CUSTOM_SETTINGS_MATCHER.match(" ".join(test_sections.description_lines)):
      sys.stderr.write('Unexpected custom settings for %s\n' % test_number)
      sys.exit(1)

    # Verify that the test is using only default settings.
    if not USING_DEFAULT_SETTINGS_MATCHER.match(
        " ".join(test_sections.procedure_lines)):
      sys.stderr.write('Unexpected procedure for %s: %s\n' %
                       (test_number, " ".join(test_section.procedure_lines)))
      sys.exit(1)

    # Check whether expected result is validation success or failure.
    result_match = TEST_RESULT_MATCHER.match(
       test_sections.expected_result_lines[0])
    if not result_match:
      sys.stderr.write('Unknown expectation for %s:\n%s\n' % (
          test_number, " ".join(test_sections.expected_result_lines)))
      sys.exit(1)
    # Initializes with default settings.
    info = TestInfo(result_match.group(1) == 'should validate')

    # Special case the 4.9 test failures (require explicit policy) to set
    # user_constrained_policy_set to empty. This is only done for the 4.9
    # tests, because the other policy tests are special cased as overrides and
    # hence set this manually on a per-test basis.
    #
    # user_constrained_policy_set enumerates the subset of the initial policy
    # set (anyPolicy in the default case) that were valid for the path. For
    # non-policy tests the expectation for user_constrained_policy_set is
    # [TEST_POLICY_1] since each policy asserts that. However for these tests,
    # the expectation is an empty user_constrained_policy_set since there was
    # no valid policy for the path (in fact, that is why the path validation is
    # expected to fail).
    if test_number.startswith('4.9.') and not info.should_validate:
      info.user_constrained_policy_set = []

    output_test(test_case_name, test_number, test_name, None, info, certs,
                crls, sanitized_test_names, output)
  else:
    # The overrides may have a series of inputs (settings) and outputs
    # (success/failure) for this test. Output each as a separate test case.
    for subpart_i in range(len(overrides)):
      info = overrides[subpart_i]
      # If the test has only 1 subpart, don't number it.
      subpart_number = subpart_i + 1 if len(overrides) > 1 else None
      output_test(test_case_name, test_number, test_name, subpart_number, info,
                  certs, crls, sanitized_test_names, output)

  return i


def main():
  pkits_pdf_path, output_path = sys.argv[1:]

  pkits_txt_file = tempfile.NamedTemporaryFile()

  subprocess.check_call(['pdftotext', '-layout', '-nopgbrk', '-eol', 'unix',
                         pkits_pdf_path, pkits_txt_file.name])

  test_descriptions = pkits_txt_file.read().decode('utf-8')

  # Extract section 4 of the text, which is the part that contains the tests.
  test_descriptions = test_descriptions.split(
      '4 Certification Path Validation Tests')[-1]
  test_descriptions = test_descriptions.split(
      '5 Relationship to Previous Test Suite', 1)[0]

  output = open(output_path, 'w')
  output.write('// Autogenerated by %s, do not edit\n\n' % sys.argv[0])
  output.write("""
// This file intentionally does not have header guards, it's intended to
// be inlined in another header file. The following line silences a
// presubmit warning that would otherwise be triggered by this:
// no-include-guard-because-multiply-included
// NOLINT(build/header_guard)\n\n""")
  output.write('// Hack to allow disabling type parameterized test cases.\n'
               '// See https://github.com/google/googletest/issues/389\n')
  output.write('#define WRAPPED_TYPED_TEST_P(CaseName, TestName) '
               'TYPED_TEST_P(CaseName, TestName)\n')
  output.write('#define WRAPPED_REGISTER_TYPED_TEST_SUITE_P(CaseName, ...) '
               'REGISTER_TYPED_TEST_SUITE_P(CaseName, __VA_ARGS__)\n\n')

  test_case_name = None
  sanitized_test_names = []

  lines = test_descriptions.splitlines()

  i = 0
  while i < len(lines):
    section_match = SECTION_MATCHER.match(lines[i])
    match = TEST_MATCHER.match(lines[i])
    i += 1

    if section_match:
      if test_case_name:
        finalize_test_case(test_case_name, sanitized_test_names, output)
        sanitized_test_names = []

      test_case_name = 'PkitsTest%02d%s' % (
          int(section_match.group(1).split('.')[-1]),
          sanitize_name(section_match.group(2)))
      output.write('\ntemplate <typename PkitsTestDelegate>\n')
      output.write('class %s : public PkitsTest<PkitsTestDelegate> {};\n' %
                   test_case_name)
      output.write('TYPED_TEST_SUITE_P(%s);\n' % test_case_name)

    if match:
      test_number = match.group(1)
      test_name = match.group(2)
      if not test_case_name:
        output.write('// Skipped %s %s\n' % (test_number, test_name))
        continue
      i, parse_test(lines, i, test_case_name, test_number,
                    test_name, sanitized_test_names, output)

  if test_case_name:
    finalize_test_case(test_case_name, sanitized_test_names, output)


if __name__ == '__main__':
  main()



# Copyright 2023 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import logging

from absl import flags
from absl.testing import absltest

from framework import xds_gamma_testcase
from framework import xds_k8s_testcase

logger = logging.getLogger(__name__)
flags.adopt_module_key_flags(xds_k8s_testcase)

_XdsTestServer = xds_k8s_testcase.XdsTestServer
_XdsTestClient = xds_k8s_testcase.XdsTestClient


class GammaBaselineTest(xds_gamma_testcase.GammaXdsKubernetesTestCase):
    def test_ping_pong(self):
        # TODO(sergiitk): [GAMMA] Consider moving out custom gamma
        #   resource creation out of self.startTestServers()
        with self.subTest("1_run_test_server"):
            test_server: _XdsTestServer = self.startTestServers()[0]

        with self.subTest("2_start_test_client"):
            test_client: _XdsTestClient = self.startTestClient(test_server)

        with self.subTest("3_test_server_received_rpcs_from_test_client"):
            self.assertSuccessfulRpcs(test_client)


if __name__ == "__main__":
    absltest.main()



#!/usr/bin/env python
# Copyright 2015 The Chromium Authors
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""Certificate chain where the target certificate is signed using a weak RSA
key (512-bit modulus)."""

import sys
sys.path += ['../..']

import gencerts

# Self-signed root certificate.
root = gencerts.create_self_signed_root_certificate('Root')

# Intermediate with a very weak key size (512-bit RSA).
intermediate = gencerts.create_intermediate_certificate('Intermediate', root)
intermediate.set_key(gencerts.get_or_generate_rsa_key(
    512, gencerts.create_key_path(intermediate.name)))

# Target certificate.
target = gencerts.create_end_entity_certificate('Target', intermediate)

chain = [target, intermediate, root]
gencerts.write_chain(__doc__, chain, 'chain.pem')



#!/usr/bin/env python3
#
# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import math

from scipy import stats

_DEFAULT_THRESHOLD = 1e-10


def scale(a, mul):
    return [x * mul for x in a]


def cmp(a, b):
    return stats.ttest_ind(a, b)


def speedup(new, old, threshold=_DEFAULT_THRESHOLD):
    if (len(set(new))) == 1 and new == old:
        return 0
    s0, p0 = cmp(new, old)
    if math.isnan(p0):
        return 0
    if s0 == 0:
        return 0
    if p0 > threshold:
        return 0
    if s0 < 0:
        pct = 1
        while pct < 100:
            sp, pp = cmp(new, scale(old, 1 - pct / 100.0))
            if sp > 0:
                break
            if pp > threshold:
                break
            pct += 1
        return -(pct - 1)
    else:
        pct = 1
        while pct < 10000:
            sp, pp = cmp(new, scale(old, 1 + pct / 100.0))
            if sp < 0:
                break
            if pp > threshold:
                break
            pct += 1
        return pct - 1


if __name__ == "__main__":
    new = [0.0, 0.0, 0.0, 0.0]
    old = [2.96608e-06, 3.35076e-06, 3.45384e-06, 3.34407e-06]
    print(speedup(new, old, 1e-5))
    print(speedup(old, new, 1e-5))



#!/usr/bin/env python3
#
# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
""" Computes the diff between two bm runs and outputs significant results """

import argparse
import collections
import json
import os
import subprocess
import sys

sys.path.append(os.path.join(os.path.dirname(sys.argv[0]), ".."))

import bm_constants
import bm_json
import bm_speedup
import tabulate

verbose = False


def _median(ary):
    assert len(ary)
    ary = sorted(ary)
    n = len(ary)
    if n % 2 == 0:
        return (ary[(n - 1) // 2] + ary[(n - 1) // 2 + 1]) / 2.0
    else:
        return ary[n // 2]


def _args():
    argp = argparse.ArgumentParser(
        description="Perform diff on microbenchmarks"
    )
    argp.add_argument(
        "-t",
        "--track",
        choices=sorted(bm_constants._INTERESTING),
        nargs="+",
        default=sorted(bm_constants._INTERESTING),
        help="Which metrics to track",
    )
    argp.add_argument(
        "-b",
        "--benchmarks",
        nargs="+",
        choices=bm_constants._AVAILABLE_BENCHMARK_TESTS,
        default=bm_constants._AVAILABLE_BENCHMARK_TESTS,
        help="Which benchmarks to run",
    )
    argp.add_argument(
        "-l",
        "--loops",
        type=int,
        default=20,
        help=(
            "Number of times to loops the benchmarks. Must match what was"
            " passed to bm_run.py"
        ),
    )
    argp.add_argument(
        "-r",
        "--regex",
        type=str,
        default="",
        help="Regex to filter benchmarks run",
    )
    argp.add_argument("-n", "--new", type=str, help="New benchmark name")
    argp.add_argument("-o", "--old", type=str, help="Old benchmark name")
    argp.add_argument(
        "-v", "--verbose", type=bool, help="Print details of before/after"
    )
    args = argp.parse_args()
    global verbose
    if args.verbose:
        verbose = True
    assert args.new
    assert args.old
    return args


def _maybe_print(str):
    if verbose:
        print(str)


class Benchmark:
    def __init__(self):
        self.samples = {
            True: collections.defaultdict(list),
            False: collections.defaultdict(list),
        }
        self.final = {}
        self.speedup = {}

    def add_sample(self, track, data, new):
        for f in track:
            if f in data:
                self.samples[new][f].append(float(data[f]))

    def process(self, track, new_name, old_name):
        for f in sorted(track):
            new = self.samples[True][f]
            old = self.samples[False][f]
            if not new or not old:
                continue
            mdn_diff = abs(_median(new) - _median(old))
            _maybe_print(
                "%s: %s=%r %s=%r mdn_diff=%r"
                % (f, new_name, new, old_name, old, mdn_diff)
            )
            s = bm_speedup.speedup(new, old, 1e-5)
            self.speedup[f] = s
            if abs(s) > 3:
                if mdn_diff > 0.5:
                    self.final[f] = "%+d%%" % s
        return self.final.keys()

    def skip(self):
        return not self.final

    def row(self, flds):
        return [self.final[f] if f in self.final else "" for f in flds]

    def speedup(self, name):
        if name in self.speedup:
            return self.speedup[name]
        return None


def _read_json(filename, badjson_files, nonexistant_files):
    stripped = ".".join(filename.split(".")[:-2])
    try:
        with open(filename) as f:
            r = f.read()
            return json.loads(r)
    except IOError as e:
        if stripped in nonexistant_files:
            nonexistant_files[stripped] += 1
        else:
            nonexistant_files[stripped] = 1
        return None
    except ValueError as e:
        print(r)
        if stripped in badjson_files:
            badjson_files[stripped] += 1
        else:
            badjson_files[stripped] = 1
        return None


def fmt_dict(d):
    return "".join(["    " + k + ": " + str(d[k]) + "\n" for k in d])


def diff(bms, loops, regex, track, old, new):
    benchmarks = collections.defaultdict(Benchmark)

    badjson_files = {}
    nonexistant_files = {}
    for bm in bms:
        for loop in range(0, loops):
            for line in subprocess.check_output(
                [
                    "bm_diff_%s/opt/%s" % (old, bm),
                    "--benchmark_list_tests",
                    "--benchmark_filter=%s" % regex,
                ]
            ).splitlines():
                line = line.decode("UTF-8")
                stripped_line = (
                    line.strip()
                    .replace("/", "_")
                    .replace("<", "_")
                    .replace(">", "_")
                    .replace(", ", "_")
                )
                js_new_opt = _read_json(
                    "%s.%s.opt.%s.%d.json" % (bm, stripped_line, new, loop),
                    badjson_files,
                    nonexistant_files,
                )
                js_old_opt = _read_json(
                    "%s.%s.opt.%s.%d.json" % (bm, stripped_line, old, loop),
                    badjson_files,
                    nonexistant_files,
                )
                if js_new_opt:
                    for row in bm_json.expand_json(js_new_opt):
                        name = row["cpp_name"]
                        if name.endswith("_mean") or name.endswith("_stddev"):
                            continue
                        benchmarks[name].add_sample(track, row, True)
                if js_old_opt:
                    for row in bm_json.expand_json(js_old_opt):
                        name = row["cpp_name"]
                        if name.endswith("_mean") or name.endswith("_stddev"):
                            continue
                        benchmarks[name].add_sample(track, row, False)

    really_interesting = set()
    for name, bm in benchmarks.items():
        _maybe_print(name)
        really_interesting.update(bm.process(track, new, old))
    fields = [f for f in track if f in really_interesting]

    # figure out the significance of the changes... right now we take the 95%-ile
    # benchmark delta %-age, and then apply some hand chosen thresholds
    histogram = []
    _NOISY = ["BM_WellFlushed"]
    for name, bm in benchmarks.items():
        if name in _NOISY:
            print(
                "skipping noisy benchmark '%s' for labelling evaluation" % name
            )
        if bm.skip():
            continue
        d = bm.speedup["cpu_time"]
        if d is None:
            continue
        histogram.append(d)
    histogram.sort()
    print("histogram of speedups: ", histogram)
    if len(histogram) == 0:
        significance = 0
    else:
        delta = histogram[int(len(histogram) * 0.95)]
        mul = 1
        if delta < 0:
            delta = -delta
            mul = -1
        if delta < 2:
            significance = 0
        elif delta < 5:
            significance = 1
        elif delta < 10:
            significance = 2
        else:
            significance = 3
        significance *= mul

    headers = ["Benchmark"] + fields
    rows = []
    for name in sorted(benchmarks.keys()):
        if benchmarks[name].skip():
            continue
        rows.append([name] + benchmarks[name].row(fields))
    note = None
    if len(badjson_files):
        note = (
            "Corrupt JSON data (indicates timeout or crash): \n%s"
            % fmt_dict(badjson_files)
        )
    if len(nonexistant_files):
        if note:
            note += (
                "\n\nMissing files (indicates new benchmark): \n%s"
                % fmt_dict(nonexistant_files)
            )
        else:
            note = (
                "\n\nMissing files (indicates new benchmark): \n%s"
                % fmt_dict(nonexistant_files)
            )
    if rows:
        return (
            tabulate.tabulate(rows, headers=headers, floatfmt="+.2f"),
            note,
            significance,
        )
    else:
        return None, note, 0


if __name__ == "__main__":
    args = _args()
    diff, note = diff(
        args.benchmarks,
        args.loops,
        args.regex,
        args.track,
        args.old,
        args.new,
        args.counters,
    )
    print("%s\n%s" % (note, diff if diff else "No performance differences"))



# Copyright 2020 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Run test xds client.

Gamma example:
./run.sh bin/run_test_client.py --server_xds_host=psm-grpc-server \
    --server_xds_port=80 \
    --config_mesh=gketd-psm-grpc-server
"""


import logging
import signal

from absl import app
from absl import flags

from bin.lib import common
from framework import xds_flags
from framework import xds_k8s_flags
from framework.infrastructure import gcp
from framework.infrastructure import k8s

logger = logging.getLogger(__name__)
# Flags
_CMD = flags.DEFINE_enum(
    "cmd", default="run", enum_values=["run", "cleanup"], help="Command"
)
_MODE = flags.DEFINE_enum(
    "mode",
    default="default",
    enum_values=[
        "default",
        "secure",
        # Uncomment if gamma-specific changes added to the client.
        # "gamma",
    ],
    help="Select client mode",
)
_QPS = flags.DEFINE_integer("qps", default=25, help="Queries per second")
_PRINT_RESPONSE = flags.DEFINE_bool(
    "print_response", default=False, help="Client prints responses"
)
_FOLLOW = flags.DEFINE_bool(
    "follow",
    default=False,
    help=(
        "Follow pod logs. Requires --collect_app_logs or"
        " --debug_use_port_forwarding"
    ),
)
_CONFIG_MESH = flags.DEFINE_string(
    "config_mesh",
    default=None,
    help="Optional. Supplied to bootstrap generator to indicate AppNet mesh.",
)
_REUSE_NAMESPACE = flags.DEFINE_bool(
    "reuse_namespace", default=True, help="Use existing namespace if exists"
)
_CLEANUP_NAMESPACE = flags.DEFINE_bool(
    "cleanup_namespace",
    default=False,
    help="Delete namespace during resource cleanup",
)
flags.adopt_module_key_flags(xds_flags)
flags.adopt_module_key_flags(xds_k8s_flags)
# Running outside of a test suite, so require explicit resource_suffix.
flags.mark_flag_as_required(xds_flags.RESOURCE_SUFFIX.name)


@flags.multi_flags_validator(
    (xds_flags.SERVER_XDS_PORT.name, _CMD.name),
    message=(
        "Run outside of a test suite, must provide"
        " the exact port value (must be greater than 0)."
    ),
)
def _check_server_xds_port_flag(flags_dict):
    if flags_dict[_CMD.name] == "cleanup":
        return True
    return flags_dict[xds_flags.SERVER_XDS_PORT.name] > 0


def _make_sigint_handler(client_runner: common.KubernetesClientRunner):
    def sigint_handler(sig, frame):
        del sig, frame
        print("Caught Ctrl+C. Shutting down the logs")
        client_runner.stop_pod_dependencies(log_drain_sec=3)

    return sigint_handler


def main(argv):
    if len(argv) > 1:
        raise app.UsageError("Too many command-line arguments.")

    # Must be called before KubernetesApiManager or GcpApiManager init.
    xds_flags.set_socket_default_timeout_from_flag()

    # Log following and port forwarding.
    should_follow_logs = _FOLLOW.value and xds_flags.COLLECT_APP_LOGS.value
    should_port_forward = (
        should_follow_logs and xds_k8s_flags.DEBUG_USE_PORT_FORWARDING.value
    )

    # Setup.
    gcp_api_manager = gcp.api.GcpApiManager()
    k8s_api_manager = k8s.KubernetesApiManager(xds_k8s_flags.KUBE_CONTEXT.value)
    client_namespace = common.make_client_namespace(k8s_api_manager)
    client_runner = common.make_client_runner(
        client_namespace,
        gcp_api_manager,
        reuse_namespace=_REUSE_NAMESPACE.value,
        mode=_MODE.value,
        port_forwarding=should_port_forward,
    )

    # Server target
    server_target = f"xds:///{xds_flags.SERVER_XDS_HOST.value}"
    if xds_flags.SERVER_XDS_PORT.value != 80:
        server_target = f"{server_target}:{xds_flags.SERVER_XDS_PORT.value}"

    if _CMD.value == "run":
        logger.info("Run client, mode=%s", _MODE.value)
        client_runner.run(
            server_target=server_target,
            qps=_QPS.value,
            print_response=_PRINT_RESPONSE.value,
            secure_mode=_MODE.value == "secure",
            config_mesh=_CONFIG_MESH.value,
            log_to_stdout=_FOLLOW.value,
        )
        if should_follow_logs:
            print("Following pod logs. Press Ctrl+C top stop")
            signal.signal(signal.SIGINT, _make_sigint_handler(client_runner))
            signal.pause()

    elif _CMD.value == "cleanup":
        logger.info("Cleanup client")
        client_runner.cleanup(
            force=True, force_namespace=_CLEANUP_NAMESPACE.value
        )


if __name__ == "__main__":
    app.run(main)




import cython

from .._cache cimport DNSCache
from .._protocol.outgoing cimport DNSOutgoing, DNSPointer, DNSQuestion, DNSRecord
from .._updates cimport RecordUpdateListener
from .._utils.time cimport current_time_millis, millis_to_seconds


cdef object TYPE_CHECKING
cdef object cached_possible_types
cdef cython.uint _EXPIRE_REFRESH_TIME_PERCENT
cdef object SERVICE_STATE_CHANGE_ADDED, SERVICE_STATE_CHANGE_REMOVED, SERVICE_STATE_CHANGE_UPDATED

cdef class _DNSPointerOutgoingBucket:

    cdef public object now
    cdef public DNSOutgoing out
    cdef public cython.uint bytes

    cpdef add(self, cython.uint max_compressed_size, DNSQuestion question, cython.set answers)


@cython.locals(answer=DNSPointer)
cdef _group_ptr_queries_with_known_answers(object now, object multicast, cython.dict question_with_known_answers)

cdef class QueryScheduler:

    cdef cython.set _types
    cdef cython.dict _next_time
    cdef object _first_random_delay_interval
    cdef cython.dict _delay

    cpdef millis_to_wait(self, object now)

    cpdef reschedule_type(self, object type_, object next_time)

    cpdef process_ready_types(self, object now)

cdef class _ServiceBrowserBase(RecordUpdateListener):

    cdef public cython.set types
    cdef public object zc
    cdef object _loop
    cdef public object addr
    cdef public object port
    cdef public object multicast
    cdef public object question_type
    cdef public cython.dict _pending_handlers
    cdef public object _service_state_changed
    cdef public QueryScheduler query_scheduler
    cdef public bint done
    cdef public object _first_request
    cdef public object _next_send_timer
    cdef public object _query_sender_task

    cpdef _generate_ready_queries(self, object first_request, object now)

    cpdef _enqueue_callback(self, object state_change, object type_, object name)

    @cython.locals(record=DNSRecord, cache=DNSCache, service=DNSRecord)
    cpdef async_update_records(self, object zc, cython.float now, cython.list records)

    cpdef _names_matching_types(self, object types)

    cpdef reschedule_type(self, object type_, object now, object next_time)

    cpdef _fire_service_state_changed_event(self, cython.tuple event)

    cpdef _async_send_ready_queries_schedule_next(self)

    cpdef _async_schedule_next(self, object now)

    cpdef _async_send_ready_queries(self, object now)

    cpdef _cancel_send_timer(self)

    cpdef async_update_records_complete(self)



# Copyright 2022 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import logging
from typing import Tuple

from absl import flags
from absl.testing import absltest

from framework import xds_url_map_testcase
from framework.test_app import client_app

# Type aliases
HostRule = xds_url_map_testcase.HostRule
PathMatcher = xds_url_map_testcase.PathMatcher
GcpResourceManager = xds_url_map_testcase.GcpResourceManager
DumpedXdsConfig = xds_url_map_testcase.DumpedXdsConfig
RpcTypeUnaryCall = xds_url_map_testcase.RpcTypeUnaryCall
RpcTypeEmptyCall = xds_url_map_testcase.RpcTypeEmptyCall
XdsTestClient = client_app.XdsTestClient

logger = logging.getLogger(__name__)
flags.adopt_module_key_flags(xds_url_map_testcase)

_NUM_RPCS = 150
_TEST_METADATA_KEY = "xds_md"
_TEST_METADATA_VALUE_EMPTY = "empty_ytpme"
_TEST_METADATA = (
    (RpcTypeEmptyCall, _TEST_METADATA_KEY, _TEST_METADATA_VALUE_EMPTY),
)
match_labels = [
    {"name": "TRAFFICDIRECTOR_NETWORK_NAME", "value": "default-vpc"}
]
not_match_labels = [{"name": "fake", "value": "fail"}]


class TestMetadataFilterMatchAll(xds_url_map_testcase.XdsUrlMapTestCase):
    """ " The test url-map has two routeRules: the higher priority routes to
    the default backends, but is supposed to be filtered out by TD because
    of non-matching metadata filters. The lower priority routes to alternative
    backends and metadata filter matches. Thus, it verifies that TD evaluates
    metadata filters correctly."""

    @staticmethod
    def url_map_change(
        host_rule: HostRule, path_matcher: PathMatcher
    ) -> Tuple[HostRule, PathMatcher]:
        path_matcher["routeRules"] = [
            {
                "priority": 0,
                "matchRules": [
                    {
                        "prefixMatch": "/",
                        "metadataFilters": [
                            {
                                "filterMatchCriteria": "MATCH_ALL",
                                "filterLabels": not_match_labels,
                            }
                        ],
                    }
                ],
                "service": GcpResourceManager().default_backend_service(),
            },
            {
                "priority": 1,
                "matchRules": [
                    {
                        "prefixMatch": "/grpc.testing.TestService/Empty",
                        "headerMatches": [
                            {
                                "headerName": _TEST_METADATA_KEY,
                                "exactMatch": _TEST_METADATA_VALUE_EMPTY,
                            }
                        ],
                        "metadataFilters": [
                            {
                                "filterMatchCriteria": "MATCH_ALL",
                                "filterLabels": match_labels,
                            }
                        ],
                    }
                ],
                "service": GcpResourceManager().alternative_backend_service(),
            },
        ]
        return host_rule, path_matcher

    def xds_config_validate(self, xds_config: DumpedXdsConfig):
        self.assertNumEndpoints(xds_config, 2)
        self.assertEqual(len(xds_config.rds["virtualHosts"][0]["routes"]), 2)
        self.assertEqual(
            xds_config.rds["virtualHosts"][0]["routes"][0]["match"]["prefix"],
            "/grpc.testing.TestService/Empty",
        )
        self.assertEqual(
            xds_config.rds["virtualHosts"][0]["routes"][0]["match"]["headers"][
                0
            ]["name"],
            _TEST_METADATA_KEY,
        )
        self.assertEqual(
            xds_config.rds["virtualHosts"][0]["routes"][0]["match"]["headers"][
                0
            ]["exactMatch"],
            _TEST_METADATA_VALUE_EMPTY,
        )
        self.assertEqual(
            xds_config.rds["virtualHosts"][0]["routes"][1]["match"]["prefix"],
            "",
        )

    def rpc_distribution_validate(self, test_client: XdsTestClient):
        rpc_distribution = self.configure_and_send(
            test_client,
            rpc_types=[RpcTypeEmptyCall],
            metadata=_TEST_METADATA,
            num_rpcs=_NUM_RPCS,
        )
        self.assertEqual(
            _NUM_RPCS, rpc_distribution.empty_call_alternative_service_rpc_count
        )


class TestMetadataFilterMatchAny(xds_url_map_testcase.XdsUrlMapTestCase):
    @staticmethod
    def url_map_change(
        host_rule: HostRule, path_matcher: PathMatcher
    ) -> Tuple[HostRule, PathMatcher]:
        path_matcher["routeRules"] = [
            {
                "priority": 0,
                "matchRules": [
                    {
                        "prefixMatch": "/",
                        "metadataFilters": [
                            {
                                "filterMatchCriteria": "MATCH_ANY",
                                "filterLabels": not_match_labels,
                            }
                        ],
                    }
                ],
                "service": GcpResourceManager().default_backend_service(),
            },
            {
                "priority": 1,
                "matchRules": [
                    {
                        "prefixMatch": "/grpc.testing.TestService/Unary",
                        "metadataFilters": [
                            {
                                "filterMatchCriteria": "MATCH_ANY",
                                "filterLabels": not_match_labels + match_labels,
                            }
                        ],
                    }
                ],
                "service": GcpResourceManager().alternative_backend_service(),
            },
        ]
        return host_rule, path_matcher

    def xds_config_validate(self, xds_config: DumpedXdsConfig):
        self.assertNumEndpoints(xds_config, 2)
        self.assertEqual(len(xds_config.rds["virtualHosts"][0]["routes"]), 2)
        self.assertEqual(
            xds_config.rds["virtualHosts"][0]["routes"][0]["match"]["prefix"],
            "/grpc.testing.TestService/Unary",
        )
        self.assertEqual(
            xds_config.rds["virtualHosts"][0]["routes"][1]["match"]["prefix"],
            "",
        )

    def rpc_distribution_validate(self, test_client: XdsTestClient):
        rpc_distribution = self.configure_and_send(
            test_client, rpc_types=(RpcTypeUnaryCall,), num_rpcs=_NUM_RPCS
        )
        self.assertEqual(
            _NUM_RPCS, rpc_distribution.unary_call_alternative_service_rpc_count
        )


class TestMetadataFilterMatchAnyAndAll(xds_url_map_testcase.XdsUrlMapTestCase):
    @staticmethod
    def url_map_change(
        host_rule: HostRule, path_matcher: PathMatcher
    ) -> Tuple[HostRule, PathMatcher]:
        path_matcher["routeRules"] = [
            {
                "priority": 0,
                "matchRules": [
                    {
                        "prefixMatch": "/",
                        "metadataFilters": [
                            {
                                "filterMatchCriteria": "MATCH_ALL",
                                "filterLabels": not_match_labels + match_labels,
                            }
                        ],
                    }
                ],
                "service": GcpResourceManager().default_backend_service(),
            },
            {
                "priority": 1,
                "matchRules": [
                    {
                        "prefixMatch": "/grpc.testing.TestService/Unary",
                        "metadataFilters": [
                            {
                                "filterMatchCriteria": "MATCH_ANY",
                                "filterLabels": not_match_labels + match_labels,
                            }
                        ],
                    }
                ],
                "service": GcpResourceManager().alternative_backend_service(),
            },
        ]
        return host_rule, path_matcher

    def xds_config_validate(self, xds_config: DumpedXdsConfig):
        self.assertNumEndpoints(xds_config, 2)
        self.assertEqual(len(xds_config.rds["virtualHosts"][0]["routes"]), 2)
        self.assertEqual(
            xds_config.rds["virtualHosts"][0]["routes"][0]["match"]["prefix"],
            "/grpc.testing.TestService/Unary",
        )
        self.assertEqual(
            xds_config.rds["virtualHosts"][0]["routes"][1]["match"]["prefix"],
            "",
        )

    def rpc_distribution_validate(self, test_client: XdsTestClient):
        rpc_distribution = self.configure_and_send(
            test_client, rpc_types=(RpcTypeUnaryCall,), num_rpcs=_NUM_RPCS
        )
        self.assertEqual(
            _NUM_RPCS, rpc_distribution.unary_call_alternative_service_rpc_count
        )


class TestMetadataFilterMatchMultipleRules(
    xds_url_map_testcase.XdsUrlMapTestCase
):
    @staticmethod
    def url_map_change(
        host_rule: HostRule, path_matcher: PathMatcher
    ) -> Tuple[HostRule, PathMatcher]:
        path_matcher["routeRules"] = [
            {
                "priority": 0,
                "matchRules": [
                    {
                        "prefixMatch": "/",
                        "headerMatches": [
                            {
                                "headerName": _TEST_METADATA_KEY,
                                "exactMatch": _TEST_METADATA_VALUE_EMPTY,
                            }
                        ],
                        "metadataFilters": [
                            {
                                "filterMatchCriteria": "MATCH_ANY",
                                "filterLabels": match_labels,
                            }
                        ],
                    }
                ],
                "service": GcpResourceManager().alternative_backend_service(),
            },
            {
                "priority": 1,
                "matchRules": [
                    {
                        "prefixMatch": "/grpc.testing.TestService/Unary",
                        "metadataFilters": [
                            {
                                "filterMatchCriteria": "MATCH_ALL",
                                "filterLabels": match_labels,
                            }
                        ],
                    }
                ],
                "service": GcpResourceManager().default_backend_service(),
            },
        ]
        return host_rule, path_matcher

    def xds_config_validate(self, xds_config: DumpedXdsConfig):
        self.assertNumEndpoints(xds_config, 2)
        self.assertEqual(len(xds_config.rds["virtualHosts"][0]["routes"]), 3)
        self.assertEqual(
            xds_config.rds["virtualHosts"][0]["routes"][0]["match"]["headers"][
                0
            ]["name"],
            _TEST_METADATA_KEY,
        )
        self.assertEqual(
            xds_config.rds["virtualHosts"][0]["routes"][0]["match"]["headers"][
                0
            ]["exactMatch"],
            _TEST_METADATA_VALUE_EMPTY,
        )
        self.assertEqual(
            xds_config.rds["virtualHosts"][0]["routes"][1]["match"]["prefix"],
            "/grpc.testing.TestService/Unary",
        )
        self.assertEqual(
            xds_config.rds["virtualHosts"][0]["routes"][2]["match"]["prefix"],
            "",
        )

    def rpc_distribution_validate(self, test_client: XdsTestClient):
        rpc_distribution = self.configure_and_send(
            test_client,
            rpc_types=[RpcTypeEmptyCall],
            metadata=_TEST_METADATA,
            num_rpcs=_NUM_RPCS,
        )
        self.assertEqual(
            _NUM_RPCS, rpc_distribution.empty_call_alternative_service_rpc_count
        )


if __name__ == "__main__":
    absltest.main()



#!/usr/bin/env python3
# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Generates the appropriate JSON data for LB interop test scenarios."""

import json
import os

import yaml

all_scenarios = []

# TODO(https://github.com/grpc/grpc-go/issues/2347): enable
# client_falls_back_because_no_backends_* scenarios for Java/Go.

# TODO(https://github.com/grpc/grpc-java/issues/4887): enable
# *short_stream* scenarios for Java.

# TODO(https://github.com/grpc/grpc-java/issues/4912): enable
# Java TLS tests involving TLS to the balancer.


def server_sec(transport_sec):
    if transport_sec == "google_default_credentials":
        return "alts", "alts", "tls"
    return transport_sec, transport_sec, transport_sec


def generate_no_balancer_because_lb_a_record_returns_nx_domain():
    all_configs = []
    for transport_sec in [
        "insecure",
        "alts",
        "tls",
        "google_default_credentials",
    ]:
        balancer_sec, backend_sec, fallback_sec = server_sec(transport_sec)
        config = {
            "name": "no_balancer_because_lb_a_record_returns_nx_domain_%s"
            % transport_sec,
            "skip_langs": [],
            "transport_sec": transport_sec,
            "balancer_configs": [],
            "backend_configs": [],
            "fallback_configs": [
                {
                    "transport_sec": fallback_sec,
                }
            ],
            "cause_no_error_no_data_for_balancer_a_record": False,
        }
        all_configs.append(config)
    return all_configs


all_scenarios += generate_no_balancer_because_lb_a_record_returns_nx_domain()


def generate_no_balancer_because_lb_a_record_returns_no_data():
    all_configs = []
    for transport_sec in [
        "insecure",
        "alts",
        "tls",
        "google_default_credentials",
    ]:
        balancer_sec, backend_sec, fallback_sec = server_sec(transport_sec)
        config = {
            "name": "no_balancer_because_lb_a_record_returns_no_data_%s"
            % transport_sec,
            "skip_langs": [],
            "transport_sec": transport_sec,
            "balancer_configs": [],
            "backend_configs": [],
            "fallback_configs": [
                {
                    "transport_sec": fallback_sec,
                }
            ],
            "cause_no_error_no_data_for_balancer_a_record": True,
        }
        all_configs.append(config)
    return all_configs


all_scenarios += generate_no_balancer_because_lb_a_record_returns_no_data()


def generate_client_referred_to_backend():
    all_configs = []
    for balancer_short_stream in [True, False]:
        for transport_sec in [
            "insecure",
            "alts",
            "tls",
            "google_default_credentials",
        ]:
            balancer_sec, backend_sec, fallback_sec = server_sec(transport_sec)
            skip_langs = []
            if transport_sec == "tls":
                skip_langs += ["java"]
            if balancer_short_stream:
                skip_langs += ["java"]
            config = {
                "name": "client_referred_to_backend_%s_short_stream_%s"
                % (transport_sec, balancer_short_stream),
                "skip_langs": skip_langs,
                "transport_sec": transport_sec,
                "balancer_configs": [
                    {
                        "transport_sec": balancer_sec,
                        "short_stream": balancer_short_stream,
                    }
                ],
                "backend_configs": [
                    {
                        "transport_sec": backend_sec,
                    }
                ],
                "fallback_configs": [],
                "cause_no_error_no_data_for_balancer_a_record": False,
            }
            all_configs.append(config)
    return all_configs


all_scenarios += generate_client_referred_to_backend()


def generate_client_referred_to_backend_fallback_broken():
    all_configs = []
    for balancer_short_stream in [True, False]:
        for transport_sec in ["alts", "tls", "google_default_credentials"]:
            balancer_sec, backend_sec, fallback_sec = server_sec(transport_sec)
            skip_langs = []
            if transport_sec == "tls":
                skip_langs += ["java"]
            if balancer_short_stream:
                skip_langs += ["java"]
            config = {
                "name": "client_referred_to_backend_fallback_broken_%s_short_stream_%s"
                % (transport_sec, balancer_short_stream),
                "skip_langs": skip_langs,
                "transport_sec": transport_sec,
                "balancer_configs": [
                    {
                        "transport_sec": balancer_sec,
                        "short_stream": balancer_short_stream,
                    }
                ],
                "backend_configs": [
                    {
                        "transport_sec": backend_sec,
                    }
                ],
                "fallback_configs": [
                    {
                        "transport_sec": "insecure",
                    }
                ],
                "cause_no_error_no_data_for_balancer_a_record": False,
            }
            all_configs.append(config)
    return all_configs


all_scenarios += generate_client_referred_to_backend_fallback_broken()


def generate_client_referred_to_backend_multiple_backends():
    all_configs = []
    for balancer_short_stream in [True, False]:
        for transport_sec in [
            "insecure",
            "alts",
            "tls",
            "google_default_credentials",
        ]:
            balancer_sec, backend_sec, fallback_sec = server_sec(transport_sec)
            skip_langs = []
            if transport_sec == "tls":
                skip_langs += ["java"]
            if balancer_short_stream:
                skip_langs += ["java"]
            config = {
                "name": "client_referred_to_backend_multiple_backends_%s_short_stream_%s"
                % (transport_sec, balancer_short_stream),
                "skip_langs": skip_langs,
                "transport_sec": transport_sec,
                "balancer_configs": [
                    {
                        "transport_sec": balancer_sec,
                        "short_stream": balancer_short_stream,
                    }
                ],
                "backend_configs": [
                    {
                        "transport_sec": backend_sec,
                    },
                    {
                        "transport_sec": backend_sec,
                    },
                    {
                        "transport_sec": backend_sec,
                    },
                    {
                        "transport_sec": backend_sec,
                    },
                    {
                        "transport_sec": backend_sec,
                    },
                ],
                "fallback_configs": [],
                "cause_no_error_no_data_for_balancer_a_record": False,
            }
            all_configs.append(config)
    return all_configs


all_scenarios += generate_client_referred_to_backend_multiple_backends()


def generate_client_falls_back_because_no_backends():
    all_configs = []
    for balancer_short_stream in [True, False]:
        for transport_sec in [
            "insecure",
            "alts",
            "tls",
            "google_default_credentials",
        ]:
            balancer_sec, backend_sec, fallback_sec = server_sec(transport_sec)
            skip_langs = ["go", "java"]
            if transport_sec == "tls":
                skip_langs += ["java"]
            if balancer_short_stream:
                skip_langs += ["java"]
            config = {
                "name": (
                    "client_falls_back_because_no_backends_%s_short_stream_%s"
                )
                % (transport_sec, balancer_short_stream),
                "skip_langs": skip_langs,
                "transport_sec": transport_sec,
                "balancer_configs": [
                    {
                        "transport_sec": balancer_sec,
                        "short_stream": balancer_short_stream,
                    }
                ],
                "backend_configs": [],
                "fallback_configs": [
                    {
                        "transport_sec": fallback_sec,
                    }
                ],
                "cause_no_error_no_data_for_balancer_a_record": False,
            }
            all_configs.append(config)
    return all_configs


all_scenarios += generate_client_falls_back_because_no_backends()


def generate_client_falls_back_because_balancer_connection_broken():
    all_configs = []
    for transport_sec in ["alts", "tls", "google_default_credentials"]:
        balancer_sec, backend_sec, fallback_sec = server_sec(transport_sec)
        skip_langs = []
        if transport_sec == "tls":
            skip_langs = ["java"]
        config = {
            "name": "client_falls_back_because_balancer_connection_broken_%s"
            % transport_sec,
            "skip_langs": skip_langs,
            "transport_sec": transport_sec,
            "balancer_configs": [
                {
                    "transport_sec": "insecure",
                    "short_stream": False,
                }
            ],
            "backend_configs": [],
            "fallback_configs": [
                {
                    "transport_sec": fallback_sec,
                }
            ],
            "cause_no_error_no_data_for_balancer_a_record": False,
        }
        all_configs.append(config)
    return all_configs


all_scenarios += generate_client_falls_back_because_balancer_connection_broken()


def generate_client_referred_to_backend_multiple_balancers():
    all_configs = []
    for balancer_short_stream in [True, False]:
        for transport_sec in [
            "insecure",
            "alts",
            "tls",
            "google_default_credentials",
        ]:
            balancer_sec, backend_sec, fallback_sec = server_sec(transport_sec)
            skip_langs = []
            if transport_sec == "tls":
                skip_langs += ["java"]
            if balancer_short_stream:
                skip_langs += ["java"]
            config = {
                "name": "client_referred_to_backend_multiple_balancers_%s_short_stream_%s"
                % (transport_sec, balancer_short_stream),
                "skip_langs": skip_langs,
                "transport_sec": transport_sec,
                "balancer_configs": [
                    {
                        "transport_sec": balancer_sec,
                        "short_stream": balancer_short_stream,
                    },
                    {
                        "transport_sec": balancer_sec,
                        "short_stream": balancer_short_stream,
                    },
                    {
                        "transport_sec": balancer_sec,
                        "short_stream": balancer_short_stream,
                    },
                    {
                        "transport_sec": balancer_sec,
                        "short_stream": balancer_short_stream,
                    },
                    {
                        "transport_sec": balancer_sec,
                        "short_stream": balancer_short_stream,
                    },
                ],
                "backend_configs": [
                    {
                        "transport_sec": backend_sec,
                    },
                ],
                "fallback_configs": [],
                "cause_no_error_no_data_for_balancer_a_record": False,
            }
            all_configs.append(config)
    return all_configs


all_scenarios += generate_client_referred_to_backend_multiple_balancers()

print(
    (
        yaml.dump(
            {
                "lb_interop_test_scenarios": all_scenarios,
            }
        )
    )
)



#!/usr/bin/env python
# Copyright 2015 The Chromium Authors
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""Certificate chain where the intermediate has an unknown critical
extension."""

import sys
sys.path += ['../..']

import gencerts

# Self-signed root certificate.
root = gencerts.create_self_signed_root_certificate('Root')

# Intermediate that has an unknown critical extension.
intermediate = gencerts.create_intermediate_certificate('Intermediate', root)
intermediate.get_extensions().add_property('1.2.3.4',
                                           'critical,DER:01:02:03:04')

# Target certificate.
target = gencerts.create_end_entity_certificate('Target', intermediate)

chain = [target, intermediate, root]
gencerts.write_chain(__doc__, chain, 'chain.pem')



# Copyright 2018 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test of gRPC Python's interaction with the python logging module"""

import logging
import subprocess
import sys
import unittest

import grpc

INTERPRETER = sys.executable


class LoggingTest(unittest.TestCase):
    def test_logger_not_occupied(self):
        script = """if True:
            import logging

            import grpc

            if len(logging.getLogger().handlers) != 0:
                raise Exception('expected 0 logging handlers')

        """
        self._verifyScriptSucceeds(script)

    def test_handler_found(self):
        script = """if True:
            import logging

            import grpc
        """
        out, err = self._verifyScriptSucceeds(script)
        self.assertEqual(0, len(err), "unexpected output to stderr")

    def test_can_configure_logger(self):
        script = """if True:
            import logging

            import grpc
            import io

            intended_stream = io.StringIO()
            logging.basicConfig(stream=intended_stream)

            if len(logging.getLogger().handlers) != 1:
                raise Exception('expected 1 logging handler')

            if logging.getLogger().handlers[0].stream is not intended_stream:
                raise Exception('wrong handler stream')

        """
        self._verifyScriptSucceeds(script)

    def test_grpc_logger(self):
        script = """if True:
            import logging

            import grpc

            if "grpc" not in logging.Logger.manager.loggerDict:
                raise Exception('grpc logger not found')

            root_logger = logging.getLogger("grpc")
            if len(root_logger.handlers) != 1:
                raise Exception('expected 1 root logger handler')
            if not isinstance(root_logger.handlers[0], logging.NullHandler):
                raise Exception('expected logging.NullHandler')

        """
        self._verifyScriptSucceeds(script)

    def _verifyScriptSucceeds(self, script):
        process = subprocess.Popen(
            [INTERPRETER, "-c", script],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        out, err = process.communicate()
        self.assertEqual(
            0,
            process.returncode,
            "process failed with exit code %d (stdout: %s, stderr: %s)"
            % (process.returncode, out, err),
        )
        return out, err


if __name__ == "__main__":
    unittest.main(verbosity=2)



#!/usr/bin/env python3

# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Explicitly ban select functions from being used in src/core/**.

Most of these functions have internal versions that should be used instead."""

import os
import sys

os.chdir(os.path.join(os.path.dirname(sys.argv[0]), "../../.."))

# map of banned function signature to allowlist
BANNED_EXCEPT = {
    "grpc_slice_from_static_buffer(": ["src/core/lib/slice/slice.cc"],
    "grpc_resource_quota_ref(": ["src/core/lib/resource_quota/api.cc"],
    "grpc_resource_quota_unref(": [
        "src/core/lib/resource_quota/api.cc",
        "src/core/lib/surface/server.cc",
    ],
    "grpc_error_create(": [
        "src/core/lib/iomgr/error.cc",
        "src/core/lib/iomgr/error_cfstream.cc",
    ],
    "grpc_error_ref(": ["src/core/lib/iomgr/error.cc"],
    "grpc_error_unref(": ["src/core/lib/iomgr/error.cc"],
    "grpc_os_error(": [
        "src/core/lib/iomgr/error.cc",
        "src/core/lib/iomgr/error.h",
    ],
    "grpc_wsa_error(": [
        "src/core/lib/iomgr/error.cc",
        "src/core/lib/iomgr/error.h",
    ],
    "grpc_log_if_error(": [
        "src/core/lib/iomgr/error.cc",
        "src/core/lib/iomgr/error.h",
    ],
    "grpc_slice_malloc(": [
        "src/core/lib/slice/slice.cc",
        "src/core/lib/slice/slice.h",
    ],
    "grpc_call_cancel(": ["src/core/lib/surface/call.cc"],
    "grpc_channel_destroy(": [
        "src/core/lib/surface/channel.cc",
        "src/core/tsi/alts/handshaker/alts_shared_resource.cc",
    ],
    "grpc_closure_create(": [
        "src/core/lib/iomgr/closure.cc",
        "src/core/lib/iomgr/closure.h",
    ],
    "grpc_closure_init(": [
        "src/core/lib/iomgr/closure.cc",
        "src/core/lib/iomgr/closure.h",
    ],
    "grpc_closure_sched(": ["src/core/lib/iomgr/closure.cc"],
    "grpc_closure_run(": ["src/core/lib/iomgr/closure.cc"],
    "grpc_closure_list_sched(": ["src/core/lib/iomgr/closure.cc"],
    "grpc_error*": ["src/core/lib/iomgr/error.cc"],
    "grpc_error_string": ["src/core/lib/iomgr/error.cc"],
    # use grpc_core::CSlice{Ref,Unref} instead inside core
    # (or prefer grpc_core::Slice!)
    "grpc_slice_ref(": ["src/core/lib/slice/slice.cc"],
    "grpc_slice_unref(": ["src/core/lib/slice/slice.cc"],
    # std::random_device needs /dev/random which is not available on all linuxes that we support.
    # Any usage must be optional and opt-in, so that those platforms can use gRPC without problem.
    "std::random_device": [
        "src/core/ext/filters/client_channel/lb_policy/rls/rls.cc",
        "src/core/ext/filters/client_channel/resolver/google_c2p/google_c2p_resolver.cc",
    ],
    # use 'grpc_core::Crash' instead
    "GPR_ASSERT(false": [],
    # Use `std::exchange()` instead.
    "absl::exchange": [],
    # Use `std::make_unique()` instead.
    "absl::make_unique": [],
}

errors = 0
num_files = 0
for root, dirs, files in os.walk("src/core"):
    if root.startswith("src/core/tsi"):
        continue
    for filename in files:
        num_files += 1
        path = os.path.join(root, filename)
        if os.path.splitext(path)[1] not in (".h", ".cc"):
            continue
        with open(path) as f:
            text = f.read()
        for banned, exceptions in list(BANNED_EXCEPT.items()):
            if path in exceptions:
                continue
            if banned in text:
                print(('Illegal use of "%s" in %s' % (banned, path)))
                errors += 1

assert errors == 0
# This check comes about from this issue:
# https://github.com/grpc/grpc/issues/15381
# Basically, a change rendered this script useless and we did not realize it.
# This dumb check ensures that this type of issue doesn't occur again.
assert num_files > 300  # we definitely have more than 300 files



# Copyright 2023 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for stateful session affinity tests.

These utilities must be shared between test environments that configure SSA
via Kubernetes CRDs and environments that configure SSA directly through the
networkservices.googleapis.com API.
"""

import datetime
import logging
from typing import Sequence, Tuple

from framework import xds_k8s_testcase
from framework.helpers import retryers

_XdsKubernetesBaseTestCase = xds_k8s_testcase.XdsKubernetesBaseTestCase
_XdsTestServer = xds_k8s_testcase.XdsTestServer
_XdsTestClient = xds_k8s_testcase.XdsTestClient

_SET_COOKIE_MAX_WAIT_SEC = 300


def get_setcookie_headers(
    metadatas_by_peer: dict[str, "MetadataByPeer"]
) -> dict[str, str]:
    cookies = dict()
    for peer, metadatas in metadatas_by_peer.items():
        for rpc_metadatas in metadatas.rpc_metadata:
            for metadata in rpc_metadatas.metadata:
                if metadata.key.lower() == "set-cookie":
                    cookies[peer] = metadata.value
    return cookies


def assert_eventually_retrieve_cookie_and_server(
    test: _XdsKubernetesBaseTestCase,
    test_client: _XdsTestClient,
    servers: Sequence[_XdsTestServer],
) -> Tuple[str, _XdsTestServer]:
    """Retrieves the initial cookie and corresponding server.

    Given a test client and set of backends for which SSA is enabled, samples
    a single RPC from the test client to the backends, with metadata collection enabled.
    The "set-cookie" header is retrieved and its contents are returned along with the
    server to which it corresponds.

    Since SSA config is supplied as a separate resource from the Route resource,
    there will be periods of time where the SSA config may not be applied. This is
    therefore an eventually consistent function.
    """

    def _assert_retrieve_cookie_and_server():
        lb_stats = test.assertSuccessfulRpcs(test_client, 1)
        cookies = get_setcookie_headers(lb_stats.metadatas_by_peer)
        test.assertLen(cookies, 1)
        hostname = next(iter(cookies.keys()))
        cookie = cookies[hostname]

        chosen_server_candidates = tuple(
            srv for srv in servers if srv.hostname == hostname
        )
        test.assertLen(chosen_server_candidates, 1)
        chosen_server = chosen_server_candidates[0]
        return cookie, chosen_server

    retryer = retryers.constant_retryer(
        wait_fixed=datetime.timedelta(seconds=10),
        timeout=datetime.timedelta(seconds=_SET_COOKIE_MAX_WAIT_SEC),
        log_level=logging.INFO,
    )
    try:
        return retryer(_assert_retrieve_cookie_and_server)
    except retryers.RetryError as retry_error:
        logging.exception(
            "Rpcs did not go to expected servers before timeout %s",
            _SET_COOKIE_MAX_WAIT_SEC,
        )
        raise retry_error



#!/usr/bin/env python
# Copyright 2019 The Chromium Authors
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""Certificate chain where the intermediate was signed by a different
certificate with a different key. The intermediate lacks an
authorityKeyIdentifier extension as some verifiers will not try verifying with
the bogus root if the authorityKeyIdentifier does not match the root's
subjectKeyIdentifier.
"""

import sys
sys.path += ['../..']

import gencerts

# Self-signed root certificate, which actually signed the intermediate.
root = gencerts.create_self_signed_root_certificate('Root')
section = root.config.get_section('signing_ca_ext')
# Don't include authorityKeyIdentifier extension in the intermediate issued by
# the real root, otherwise the verifier might not even try to validate the
# signature against the bogus root.
section.remove_property('authorityKeyIdentifier')

# Intermediate certificate.
intermediate = gencerts.create_intermediate_certificate('Intermediate', root)

# Target certificate.
target = gencerts.create_end_entity_certificate('Target', intermediate)
# TODO(eroman): Set subjectAltName by default rather than specifically in
# this test.
target.get_extensions().set_property('subjectAltName', 'DNS:test.example')

# Self-signed root certificate that has nothing to do with this chain, but will
# be saved as its root certificate.
bogus_root = gencerts.create_self_signed_root_certificate('Root')

chain = [target, intermediate, bogus_root]
gencerts.write_chain(__doc__, chain, 'chain.pem')



#!/usr/bin/env python


"""Unit tests for zeroconf._utils.net."""
import errno
import socket
import unittest
from unittest.mock import MagicMock, Mock, patch

import ifaddr
import pytest

import zeroconf as r
from zeroconf._utils import net as netutils


def _generate_mock_adapters():
    mock_lo0 = Mock(spec=ifaddr.Adapter)
    mock_lo0.nice_name = "lo0"
    mock_lo0.ips = [ifaddr.IP("127.0.0.1", 8, "lo0")]
    mock_lo0.index = 0
    mock_eth0 = Mock(spec=ifaddr.Adapter)
    mock_eth0.nice_name = "eth0"
    mock_eth0.ips = [ifaddr.IP(("2001:db8::", 1, 1), 8, "eth0")]
    mock_eth0.index = 1
    mock_eth1 = Mock(spec=ifaddr.Adapter)
    mock_eth1.nice_name = "eth1"
    mock_eth1.ips = [ifaddr.IP("192.168.1.5", 23, "eth1")]
    mock_eth1.index = 2
    mock_vtun0 = Mock(spec=ifaddr.Adapter)
    mock_vtun0.nice_name = "vtun0"
    mock_vtun0.ips = [ifaddr.IP("169.254.3.2", 16, "vtun0")]
    mock_vtun0.index = 3
    return [mock_eth0, mock_lo0, mock_eth1, mock_vtun0]


def test_ip6_to_address_and_index():
    """Test we can extract from mocked adapters."""
    adapters = _generate_mock_adapters()
    assert netutils.ip6_to_address_and_index(adapters, "2001:db8::") == (('2001:db8::', 1, 1), 1)
    assert netutils.ip6_to_address_and_index(adapters, "2001:db8::%1") == (('2001:db8::', 1, 1), 1)
    with pytest.raises(RuntimeError):
        assert netutils.ip6_to_address_and_index(adapters, "2005:db8::")


def test_interface_index_to_ip6_address():
    """Test we can extract from mocked adapters."""
    adapters = _generate_mock_adapters()
    assert netutils.interface_index_to_ip6_address(adapters, 1) == ('2001:db8::', 1, 1)

    # call with invalid adapter
    with pytest.raises(RuntimeError):
        assert netutils.interface_index_to_ip6_address(adapters, 6)

    # call with adapter that has ipv4 address only
    with pytest.raises(RuntimeError):
        assert netutils.interface_index_to_ip6_address(adapters, 2)


def test_ip6_addresses_to_indexes():
    """Test we can extract from mocked adapters."""
    interfaces = [1]
    with patch("zeroconf._utils.net.ifaddr.get_adapters", return_value=_generate_mock_adapters()):
        assert netutils.ip6_addresses_to_indexes(interfaces) == [(('2001:db8::', 1, 1), 1)]

    interfaces_2 = ['2001:db8::']
    with patch("zeroconf._utils.net.ifaddr.get_adapters", return_value=_generate_mock_adapters()):
        assert netutils.ip6_addresses_to_indexes(interfaces_2) == [(('2001:db8::', 1, 1), 1)]


def test_normalize_interface_choice_errors():
    """Test we generate exception on invalid input."""
    with patch("zeroconf._utils.net.get_all_addresses", return_value=[]), patch(
        "zeroconf._utils.net.get_all_addresses_v6", return_value=[]
    ), pytest.raises(RuntimeError):
        netutils.normalize_interface_choice(r.InterfaceChoice.All)

    with pytest.raises(TypeError):
        netutils.normalize_interface_choice("1.2.3.4")


@pytest.mark.parametrize(
    "errno,expected_result",
    [(errno.EADDRINUSE, False), (errno.EADDRNOTAVAIL, False), (errno.EINVAL, False), (0, True)],
)
def test_add_multicast_member_socket_errors(errno, expected_result):
    """Test we handle socket errors when adding multicast members."""
    if errno:
        setsockopt_mock = unittest.mock.Mock(side_effect=OSError(errno, f"Error: {errno}"))
    else:
        setsockopt_mock = unittest.mock.Mock()
    fileno_mock = unittest.mock.PropertyMock(return_value=10)
    socket_mock = unittest.mock.Mock(setsockopt=setsockopt_mock, fileno=fileno_mock)
    assert r.add_multicast_member(socket_mock, "0.0.0.0") == expected_result


def test_autodetect_ip_version():
    """Tests for auto detecting IPVersion based on interface ips."""
    assert r.autodetect_ip_version(["1.3.4.5"]) is r.IPVersion.V4Only
    assert r.autodetect_ip_version([]) is r.IPVersion.V4Only
    assert r.autodetect_ip_version(["::1", "1.2.3.4"]) is r.IPVersion.All
    assert r.autodetect_ip_version(["::1"]) is r.IPVersion.V6Only


def test_disable_ipv6_only_or_raise():
    """Test that IPV6_V6ONLY failing logs a nice error message and still raises."""
    errors_logged = []

    def _log_error(*args):
        nonlocal errors_logged
        errors_logged.append(args)

    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    with pytest.raises(OSError), patch.object(netutils.log, "error", _log_error), patch(
        "socket.socket.setsockopt", side_effect=OSError
    ):
        netutils.disable_ipv6_only_or_raise(sock)

    assert (
        errors_logged[0][0]
        == 'Support for dual V4-V6 sockets is not present, use IPVersion.V4 or IPVersion.V6'
    )


@pytest.mark.skipif(not hasattr(socket, 'SO_REUSEPORT'), reason="System does not have SO_REUSEPORT")
def test_set_so_reuseport_if_available_is_present():
    """Test that setting socket.SO_REUSEPORT only OSError errno.ENOPROTOOPT is trapped."""
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    with pytest.raises(OSError), patch("socket.socket.setsockopt", side_effect=OSError):
        netutils.set_so_reuseport_if_available(sock)

    with patch("socket.socket.setsockopt", side_effect=OSError(errno.ENOPROTOOPT, None)):
        netutils.set_so_reuseport_if_available(sock)


@pytest.mark.skipif(hasattr(socket, 'SO_REUSEPORT'), reason="System has SO_REUSEPORT")
def test_set_so_reuseport_if_available_not_present():
    """Test that we do not try to set SO_REUSEPORT if it is not present."""
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    with patch("socket.socket.setsockopt", side_effect=OSError):
        netutils.set_so_reuseport_if_available(sock)


def test_set_mdns_port_socket_options_for_ip_version():
    """Test OSError with errno with EINVAL and bind address '' from setsockopt IP_MULTICAST_TTL does not raise."""
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

    # Should raise on EPERM always
    with pytest.raises(OSError), patch("socket.socket.setsockopt", side_effect=OSError(errno.EPERM, None)):
        netutils.set_mdns_port_socket_options_for_ip_version(sock, ('',), r.IPVersion.V4Only)

    # Should raise on EINVAL always when bind address is not ''
    with pytest.raises(OSError), patch("socket.socket.setsockopt", side_effect=OSError(errno.EINVAL, None)):
        netutils.set_mdns_port_socket_options_for_ip_version(sock, ('127.0.0.1',), r.IPVersion.V4Only)

    # Should not raise on EINVAL when bind address is ''
    with patch("socket.socket.setsockopt", side_effect=OSError(errno.EINVAL, None)):
        netutils.set_mdns_port_socket_options_for_ip_version(sock, ('',), r.IPVersion.V4Only)


def test_add_multicast_member():
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    interface = '127.0.0.1'

    # EPERM should always raise
    with pytest.raises(OSError), patch("socket.socket.setsockopt", side_effect=OSError(errno.EPERM, None)):
        netutils.add_multicast_member(sock, interface)

    # EADDRINUSE should return False
    with patch("socket.socket.setsockopt", side_effect=OSError(errno.EADDRINUSE, None)):
        assert netutils.add_multicast_member(sock, interface) is False

    # EADDRNOTAVAIL should return False
    with patch("socket.socket.setsockopt", side_effect=OSError(errno.EADDRNOTAVAIL, None)):
        assert netutils.add_multicast_member(sock, interface) is False

    # EINVAL should return False
    with patch("socket.socket.setsockopt", side_effect=OSError(errno.EINVAL, None)):
        assert netutils.add_multicast_member(sock, interface) is False

    # ENOPROTOOPT should return False
    with patch("socket.socket.setsockopt", side_effect=OSError(errno.ENOPROTOOPT, None)):
        assert netutils.add_multicast_member(sock, interface) is False

    # ENODEV should raise for ipv4
    with pytest.raises(OSError), patch("socket.socket.setsockopt", side_effect=OSError(errno.ENODEV, None)):
        netutils.add_multicast_member(sock, interface) is False

    # ENODEV should return False for ipv6
    with patch("socket.socket.setsockopt", side_effect=OSError(errno.ENODEV, None)):
        assert netutils.add_multicast_member(sock, ('2001:db8::', 1, 1)) is False  # type: ignore[arg-type]

    # No IPv6 support should return False for IPv6
    with patch("socket.inet_pton", side_effect=OSError()):
        assert netutils.add_multicast_member(sock, ('2001:db8::', 1, 1)) is False  # type: ignore[arg-type]

    # No error should return True
    with patch("socket.socket.setsockopt"):
        assert netutils.add_multicast_member(sock, interface) is True


def test_bind_raises_skips_address():
    """Test bind failing in new_socket returns None on EADDRNOTAVAIL."""
    err = errno.EADDRNOTAVAIL

    def _mock_socket(*args, **kwargs):
        sock = MagicMock()
        sock.bind = MagicMock(side_effect=OSError(err, f"Error: {err}"))
        return sock

    with patch("socket.socket", _mock_socket):
        assert netutils.new_socket(("0.0.0.0", 0)) is None  # type: ignore[arg-type]

    err = errno.EAGAIN
    with pytest.raises(OSError), patch("socket.socket", _mock_socket):
        netutils.new_socket(("0.0.0.0", 0))  # type: ignore[arg-type]


def test_new_respond_socket_new_socket_returns_none():
    """Test new_respond_socket returns None if new_socket returns None."""
    with patch.object(netutils, "new_socket", return_value=None):
        assert netutils.new_respond_socket(("0.0.0.0", 0)) is None  # type: ignore[arg-type]



#!/usr/bin/env python3
#
# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
""" Python utility to build opt and counters benchmarks """

import argparse
import multiprocessing
import os
import shutil
import subprocess

import bm_constants


def _args():
    argp = argparse.ArgumentParser(description="Builds microbenchmarks")
    argp.add_argument(
        "-b",
        "--benchmarks",
        nargs="+",
        choices=bm_constants._AVAILABLE_BENCHMARK_TESTS,
        default=bm_constants._AVAILABLE_BENCHMARK_TESTS,
        help="Which benchmarks to build",
    )
    argp.add_argument(
        "-j",
        "--jobs",
        type=int,
        default=multiprocessing.cpu_count(),
        help=(
            "Deprecated. Bazel chooses number of CPUs to build with"
            " automatically."
        ),
    )
    argp.add_argument(
        "-n",
        "--name",
        type=str,
        help=(
            "Unique name of this build. To be used as a handle to pass to the"
            " other bm* scripts"
        ),
    )
    args = argp.parse_args()
    assert args.name
    return args


def _build_cmd(cfg, benchmarks):
    bazel_targets = [
        "//test/cpp/microbenchmarks:%s" % benchmark for benchmark in benchmarks
    ]
    # --dynamic_mode=off makes sure that we get a monolithic binary that can be safely
    # moved outside of the bazel-bin directory
    return [
        "tools/bazel",
        "build",
        "--config=%s" % cfg,
        "--dynamic_mode=off",
    ] + bazel_targets


def _build_config_and_copy(cfg, benchmarks, dest_dir):
    """Build given config and copy resulting binaries to dest_dir/CONFIG"""
    subprocess.check_call(_build_cmd(cfg, benchmarks))
    cfg_dir = dest_dir + "/%s" % cfg
    os.makedirs(cfg_dir)
    subprocess.check_call(
        ["cp"]
        + [
            "bazel-bin/test/cpp/microbenchmarks/%s" % benchmark
            for benchmark in benchmarks
        ]
        + [cfg_dir]
    )


def build(name, benchmarks, jobs):
    dest_dir = "bm_diff_%s" % name
    shutil.rmtree(dest_dir, ignore_errors=True)
    _build_config_and_copy("opt", benchmarks, dest_dir)


if __name__ == "__main__":
    args = _args()
    build(args.name, args.benchmarks, args.jobs)



# Copyright 2023 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from absl import app
from absl import flags
from absl import logging

from bin.lib import common
from framework import xds_flags
from framework import xds_k8s_flags
from framework.helpers import grpc as helpers_grpc
import framework.helpers.highlighter
from framework.infrastructure import gcp
from framework.infrastructure import k8s
from framework.rpc import grpc_channelz
from framework.rpc import grpc_testing
from framework.test_app import client_app
from framework.test_app import server_app

# Flags
_MODE = flags.DEFINE_enum(
    "mode",
    default="default",
    enum_values=["default", "secure", "gamma"],
    help="Select a deployment of the client/server",
)
_NUM_RPCS = flags.DEFINE_integer(
    "num_rpcs",
    default=100,
    lower_bound=1,
    upper_bound=10_000,
    help="The number of RPCs to check.",
)
flags.adopt_module_key_flags(xds_flags)
flags.adopt_module_key_flags(xds_k8s_flags)
# Running outside of a test suite, so require explicit resource_suffix.
flags.mark_flag_as_required(xds_flags.RESOURCE_SUFFIX.name)
flags.register_validator(
    xds_flags.SERVER_XDS_PORT.name,
    lambda val: val > 0,
    message=(
        "Run outside of a test suite, must provide"
        " the exact port value (must be greater than 0)."
    ),
)

logger = logging.get_absl_logger()

# Type aliases
_Channel = grpc_channelz.Channel
_Socket = grpc_channelz.Socket
_ChannelState = grpc_channelz.ChannelState
_XdsTestServer = server_app.XdsTestServer
_XdsTestClient = client_app.XdsTestClient
LoadBalancerStatsResponse = grpc_testing.LoadBalancerStatsResponse


def get_client_rpc_stats(
    test_client: _XdsTestClient, num_rpcs: int
) -> LoadBalancerStatsResponse:
    lb_stats = test_client.get_load_balancer_stats(num_rpcs=num_rpcs)
    hl = framework.helpers.highlighter.HighlighterYaml()
    logger.info(
        "[%s] Received LoadBalancerStatsResponse:\n%s",
        test_client.hostname,
        hl.highlight(helpers_grpc.lb_stats_pretty(lb_stats)),
    )
    return lb_stats


def run_ping_pong(test_client: _XdsTestClient, num_rpcs: int):
    test_client.wait_for_active_server_channel()
    lb_stats = get_client_rpc_stats(test_client, num_rpcs)
    for backend, rpcs_count in lb_stats.rpcs_by_peer.items():
        if int(rpcs_count) < 1:
            raise AssertionError(
                f"Backend {backend} did not receive a single RPC"
            )

    failed = int(lb_stats.num_failures)
    if int(lb_stats.num_failures) > 0:
        raise AssertionError(
            f"Expected all RPCs to succeed: {failed} of {num_rpcs} failed"
        )


def main(argv):
    if len(argv) > 1:
        raise app.UsageError("Too many command-line arguments.")

    # Must be called before KubernetesApiManager or GcpApiManager init.
    xds_flags.set_socket_default_timeout_from_flag()

    # Flags.
    should_port_forward: bool = xds_k8s_flags.DEBUG_USE_PORT_FORWARDING.value

    # Setup.
    gcp_api_manager = gcp.api.GcpApiManager()
    k8s_api_manager = k8s.KubernetesApiManager(xds_k8s_flags.KUBE_CONTEXT.value)

    # Server.
    server_namespace = common.make_server_namespace(k8s_api_manager)
    server_runner = common.make_server_runner(
        server_namespace,
        gcp_api_manager,
        port_forwarding=should_port_forward,
        mode=_MODE.value,
    )
    # Find server pod.
    server_pod: k8s.V1Pod = common.get_server_pod(
        server_runner, xds_flags.SERVER_NAME.value
    )

    # Client
    client_namespace = common.make_client_namespace(k8s_api_manager)
    client_runner = common.make_client_runner(
        client_namespace,
        gcp_api_manager,
        port_forwarding=should_port_forward,
        mode=_MODE.value,
    )
    # Find client pod.
    client_pod: k8s.V1Pod = common.get_client_pod(
        client_runner, xds_flags.CLIENT_NAME.value
    )

    # Ensure port forwarding stopped.
    common.register_graceful_exit(server_runner, client_runner)

    # Create server app for the server pod.
    test_server: _XdsTestServer = common.get_test_server_for_pod(
        server_runner,
        server_pod,
        test_port=xds_flags.SERVER_PORT.value,
        secure_mode=_MODE.value == "secure",
    )
    test_server.set_xds_address(
        xds_flags.SERVER_XDS_HOST.value, xds_flags.SERVER_XDS_PORT.value
    )

    # Create client app for the client pod.
    test_client: _XdsTestClient = common.get_test_client_for_pod(
        client_runner, client_pod, server_target=test_server.xds_uri
    )

    with test_client, test_server:
        run_ping_pong(test_client, _NUM_RPCS.value)

    logger.info("SUCCESS!")


if __name__ == "__main__":
    app.run(main)
